package org.ccgv.text.clearnlp;

import gate.Gate;
import gate.util.GateException;


import org.ccgv.text.gate.GateInstance;
import org.ccgv.text.gate.util.CommonMethod;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.StringTokenizer;
import java.util.zip.ZipInputStream;


public class DepRelParser{}

/*
import com.googlecode.clearnlp.component.AbstractComponent;
import com.googlecode.clearnlp.component.dep.AbstractDEPParser;
import com.googlecode.clearnlp.dependency.DEPTree;
import com.googlecode.clearnlp.engine.EngineGetter;
import com.googlecode.clearnlp.nlp.NLPDecode;
import com.googlecode.clearnlp.nlp.NLPLib;
import com.googlecode.clearnlp.reader.AbstractReader;
import com.googlecode.clearnlp.segmentation.AbstractSegmenter;
import com.googlecode.clearnlp.tokenization.AbstractTokenizer;
import com.googlecode.clearnlp.util.pair.ObjectDoublePair;
import com.googlecode.clearnlp.util.pair.StringIntPair;

public class DepRelParser {

	private static DepRelParser instance = null;
	private String path ;
	private String dictFile;
	private String posModelFile; // e.g., ontonotes-en-pos.tgz
	private String depModelFile ; // e.g., ontonotes-en-dep.tgz
	private String predModelFile; // e.g., ontonotes-en-pred.tgz
	private String roleModelFile; // e.g., ontonotes-en-role.tgz
	private String srlModelFile; // e.g., ontonotes-en-srl.tgz
	private String inputFile;
	private String outputFile;
	final String language = AbstractReader.LANG_EN;
	private static  AbstractComponent[] preComponents;
	private static  AbstractComponent[] postComponents;
	private static AbstractTokenizer tokenizer;
	private static AbstractDEPParser parser;
	
	private DepRelParser(){
		path = CommonMethod.getGateCfgPathWrite()+"clearResource"+File.separator;
		CommonMethod.screenPrint("in DepRelParser.java path:"+path);
		//path = "D:\\work\\Gate\\myProject\\gateProject\\clearResource\\";
		dictFile      = path+"dictionary-1.4.0.zip"; // e.g., dictionary.zip
		posModelFile  =   path+"ontonotes-en-pos-1.4.0.tgz"; // e.g., ontonotes-en-pos.tgz
		depModelFile  =  path+"ontonotes-en-dep-1.4.0.tgz"; // e.g., ontonotes-en-dep.tgz
		predModelFile =  path+"ontonotes-en-pred-1.4.0.tgz";; // e.g., ontonotes-en-pred.tgz
		roleModelFile =  path+"ontonotes-en-role-1.4.0.tgz";; // e.g., ontonotes-en-role.tgz
		srlModelFile  =  path+"ontonotes-en-srl-1.4.2.tgz";; // e.g., ontonotes-en-srl.tgz
		inputFile     =  path+"testIn.txt";
		outputFile    =  path+"testOut.txt";
		//---------------------------------
		init();
	}
	public static DepRelParser getInstance() throws GateException, IOException{
		if(instance==null){
			instance = new DepRelParser();
		}
		return instance;
	}
	private void init(){
		try {
            tokenizer  = EngineGetter.getTokenizer(language, new FileInputStream(dictFile));
            AbstractComponent tagger     = EngineGetter.getComponent(new FileInputStream(posModelFile) , language, NLPLib.MODE_POS);
            AbstractComponent analyzer   = EngineGetter.getComponent(new FileInputStream(dictFile)     , language, NLPLib.MODE_MORPH);
            parser     = EngineGetter.getDEPParser(new ZipInputStream(new FileInputStream(depModelFile)), language);
            AbstractComponent identifier = EngineGetter.getComponent(new FileInputStream(predModelFile), language, NLPLib.MODE_PRED);
            AbstractComponent classifier = EngineGetter.getComponent(new FileInputStream(roleModelFile), language, NLPLib.MODE_ROLE);
            AbstractComponent labeler    = EngineGetter.getComponent(new FileInputStream(srlModelFile) , language, NLPLib.MODE_SRL);
            
            //preComponents  = {tagger, analyzer};        // components used before parsing
            //postComponents = {identifier, classifier, labeler}; // components used after parsing
            preComponents = new AbstractComponent[2];
            preComponents[0] = tagger;
            preComponents[1] = analyzer;
            postComponents = new AbstractComponent[3];
            postComponents[0]= identifier;
            postComponents[1]= classifier;
            postComponents[2]= labeler;
            		
			
		} catch(IOException e){
			e.printStackTrace();
		}
	}
	
	
	public AbstractComponent[] getPreComponent(){
		return preComponents;
	}
	public  AbstractComponent[] getPostComponent(){
		return postComponents;
	}
	public AbstractTokenizer getTokenizer(){
		return tokenizer;
	}
	//
	 * dep:
	 *  1	This	this	DT	_	2	nsubj
		2	is	be	VBZ	pb=be.01	0	root
		3	Professor	professor	NNP	_	4	nn
		4	Choi	choi	NNP	_	6	poss
		5	's	's	POS	_	4	possessive
		6	lab	lab	NN	_	2	attr
		7	.	.	.	_	2	punct
		
ID: current token ID (starting at 1)
FORM: word form
LEMMA: lemma
POS: part-of-speech tag
FEATS: features (different features are delimited by '|', keys and values are delimited by '=', and '_' indicates no feature)
HEAD: head token ID
DEPREL: dependency label
SHEADS: semantic heads ('_' indicates no semantic head)
//		
    public SentenceRelationModel process(  String aSentence)
    {
    		CommonMethod.screenPrint("----------------------------the Sentence:"+aSentence+".");
    		SentenceRelationModel rtn = new SentenceRelationModel();
            DEPTree tree = NLPDecode.toDEPTree(tokenizer.getTokens(aSentence));
            
            List<ObjectDoublePair<DEPTree>> trees = getParses( tree);
            String dep = "";
            for (ObjectDoublePair<DEPTree> p : trees)
            {
                    tree = (DEPTree)p.o;
                    dep = tree.toStringDEP();  
                    //System.out.println("Score: "+p.d);
                    //System.out.println(tree.toStringSRL()+"\n");
                    //String raw = tree.toStringRaw();
                    //String pos = tree.toStringPOS();
            }
            
            CommonMethod.screenPrint("in DepRelParser dep:\n"+dep);
            StringTokenizer st = new StringTokenizer(dep,"\n");
            StringTokenizer st2 ;
            String str;
            
            while(st.hasMoreTokens()){
            	str = st.nextToken();
            	//CommonMethod.screenPrint("str:"+str);
            	SentenceWordModel aWordModel = new SentenceWordModel();
            	st2 = new StringTokenizer(str);
            	int i=0;
            	while(st2.hasMoreTokens()){
            		str = st2.nextToken();
            		if(i==0){
            			aWordModel.setId(str);
            		}
            		if(i==1){//raw
            			aWordModel.setRaw(str);
            		}else if(i==2){
            			aWordModel.setLemma(str);
            		}else if(i==3){//POS
            			aWordModel.setPos(str);
            		}else if(i==4){//FEATS
            			aWordModel.setFeat(str);
            		}else if(i==5){//HEAD
            			aWordModel.setPhead(str);
            		}
            		else if(i==6){//DEPREL
            			aWordModel.setDeprel(str);
            		}else if(i==7){//SHEADS
            			aWordModel.setSheads(str);
            		}
            		
            		i++;
            	}
            	CommonMethod.screenPrint("("+aWordModel.toString()+")");
            	rtn.getTheSentence().add(aWordModel);
            	//CommonMethod.screenPrint(" in DepRelParser process SentenceRelationModel:"+aModel.toString());
            }
           
            //System.out.println("raw:"+raw+"\n");
            //System.out.println("pos:"+pos+"\n");
            //System.out.println("dep:"+dep+"\n");
            //System.out.println();
            CommonMethod.screenPrint("DepRelParser process rtn.size()"+rtn.getTheSentence().size());
            return rtn;
    }
    

    private List<ObjectDoublePair<DEPTree>> getParses( DEPTree tree)
    {
            List<ObjectDoublePair<DEPTree>> trees;
            boolean uniqueOnly = true;      // return only unique trees given a sentence 
            
            for (AbstractComponent component : preComponents)
                    component.process(tree);
            
            parser.process(tree);
            trees = parser.getParsedTrees(uniqueOnly);      // parses are already sorted by their scores in descending order
            
            for (ObjectDoublePair<DEPTree> p : trees)
            {
                    tree = (DEPTree)p.o;
                    
                    for (AbstractComponent component : postComponents)
                            component.process(tree);
            }
            
            return trees;
    }
	
	
	
	public static void main(String args[]){
		String sentence1 = "BACKGROUND:With drug treatment for dementia being of limited effectiveness, the role of primary prevention, in particular the predictive value of modifiable cardiovascular disease risk factors, may warrant exploration.";
		//String sentence2 = "It is commonly recognized that obesity or smoking is an established risk  factor for type 2 diabetes mellitus.";
		try {
			//CommonMethod.screenPrint("in depRelParser before process:"+Calendar.getInstance().getTimeInMillis());
			DepRelParser instance = DepRelParser.getInstance();
			//CommonMethod.screenPrint("in depRelParser process 1:"+Calendar.getInstance().getTimeInMillis());
			SentenceRelationModel model= instance.process( sentence1);
			//CommonMethod.screenPrint("in depRelParser process 2:"+Calendar.getInstance().getTimeInMillis());
			//instance.process( sentence2);
			//CommonMethod.screenPrint("in depRelParser finish:"+Calendar.getInstance().getTimeInMillis());
			//time output:
			 * 1410452174895
			   1410452198553
               1410452198612
               1410452198621
			 * initialize use 24 seconds, processing use 50 milli seconds for first processing and 9 milli seconds for second processing
			 * 
			 //
		} catch (GateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
}
*/