package org.ccgv.text.gate;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import gate.*;
import gate.creole.POSTagger;
import gate.creole.ResourceInstantiationException;
import gate.creole.gazetteer.DefaultGazetteer;
import gate.creole.gazetteer.Gazetteer;
import gate.creole.morph.Morph;
import gate.creole.ontology.Ontology;
import gate.creole.tokeniser.DefaultTokeniser;
import gate.util.GateException;

import org.ccgv.text.gate.util.CommonMethod;


public class GatePRInstance {
	
	private static final String SERIAL_CONTROLLER = "gate.creole.SerialAnalyserController";
	private static final String FLEXIBLE_GAZETTEER = "gate.creole.gazetteer.FlexibleGazetteer";
	private static final String ANNIE_GAZETTEER = "gate.creole.gazetteer.DefaultGazetteer";
	private static final String ONTO_ROOT_GAZETTEER = "gate.clone.ql.OntoRootGaz";
	// private static final String ONTOLOGY_LR =
	// "gate.creole.ontology.owlim.OWLIMOntologyLR";
	private static final String ONTOLOGY_OWLIM = "gate.creole.ontology.impl.sesame.OWLIMOntology";
	private static final String JAPE_TRANSDUCER = "gate.creole.Transducer"; // alternate com.ontotext.gate.japec.JapecTransducer
	
	private static final String HASH_GAZETTEER = "com.ontotext.gate.gazetteer.HashGazetteer";//for domain dic
	
	private static final String DOCUMENT_RESET = "gate.creole.annotdelete.AnnotationDeletePR";
	private static final String REGEX_SENTENCE_SPLITTER = "gate.creole.splitter.RegexSentenceSplitter";
	private static final String ENGLISH_TOKENISER = "gate.creole.tokeniser.DefaultTokeniser";
	private static final String POS = "gate.creole.POSTagger";
	private static final String MORPHER = "gate.creole.morph.Morph";
	
	
	
	private static GatePRInstance instance = null;
	private static int mode = 0;
	//----------------PRs:
	private static ProcessingResource flexPR;
	private static ProcessingResource tokenizerPR;
	private static ProcessingResource posPR;
	private static ProcessingResource morpherPR;
	
	
	private GatePRInstance(int mode) throws ResourceInstantiationException{
		
		String pluginsHome = CommonMethod.getGatePluginsHome();
	
		
		init(mode);
	}
	public static GatePRInstance getInstance(int mode) throws GateException, IOException{
		if(instance==null){
			instance = new GatePRInstance(mode);
		}
		return instance;
	}
	
	private boolean init(int mode) throws ResourceInstantiationException{
		boolean rtn = false;
		//-----------init a FlexPR for ICD10 onto
		String ontoPath = "";//filePath
		String ICDFile = "";//fileName
		String owlFile = ontoPath + ICDFile;
		if(flexPR==null){
			flexPR = getFlexPrForICD( owlFile,mode);
		}
		//------------------------------------
		return rtn;
	}

	private ProcessingResource getFlexPrForICD(String owlFile, int mode) throws ResourceInstantiationException{
		ProcessingResource rtn = null;
		
		//-----------create ontology param
		FeatureMap paramsOnto,paramsRGPR,paramsFlexPR;
		paramsOnto = Factory.newFeatureMap(); // use default parameters
		//String filePath = getOntologyFilePathWrite(fileName);
		
		URL fUrl = null;
		try {
			if(mode ==0){
				fUrl = new URL("file:/" + owlFile);
			}else{
				fUrl = new URL( owlFile);
			}
				
		} catch (MalformedURLException e) {
			System.out.println(e);
			e.printStackTrace();
				//throw new GenericException(e.getMessage(), "Back", "dashboard");
		}
		
		paramsOnto.put("rdfXmlURL", fUrl);
		
		Ontology ontology = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,paramsOnto);
		//----------------prepare for flex param PR-------------
		if(tokenizerPR==null){
			tokenizerPR = this.getTokenizerPR();
		}
		if(posPR == null){
			posPR = this.getPosPR();
		}
		if(morpherPR == null){
			morpherPR = this.getMorphPR();
		}
		//-------------------------------------
		//------------create pr param----------------
				paramsRGPR = Factory.newFeatureMap(); // use default parameters
				paramsRGPR.put("ontology", ontology); // gate.creole.ontology.Ontology
				paramsRGPR.put("tokeniser", (DefaultTokeniser) this.tokenizerPR); // gate.creole.tokeniser.DefaultTokeniser
				paramsRGPR.put("posTagger", (POSTagger) this.posPR); // gate.creole.POSTagger
				paramsRGPR.put("morpher", (Morph) this.morpherPR); // gate.creole.morph.Morph
				paramsRGPR.put("caseSensitive", false);//be sure
				Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, paramsRGPR);
				List<String> fnames = new ArrayList<String>();
				fnames.add("Token.root");
		
		return rtn;
	}
	

	private ProcessingResource getTokenizerPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(ENGLISH_TOKENISER, params);
		return rtn;
	}
	private ProcessingResource getSentenceSplitterPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(REGEX_SENTENCE_SPLITTER, params);
		return rtn;
	}
	private ProcessingResource getPosPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(POS, params);
		return rtn;
	}
	private ProcessingResource getMorphPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(MORPHER, params);
		return rtn;
	}
	
	

	
	public Boolean init(){
		boolean rtn = false;
		//
		return rtn;
	}
	
	
	private  ProcessingResource getFlexibleGazetteer(String filePath)throws ResourceInstantiationException
	{	
		ProcessingResource rtn = null;
		//-----------create ontology param
		FeatureMap paramsOnto,paramsRootGaztPR,paramsFlexPR;
		paramsOnto = Factory.newFeatureMap(); // use default parameters
		//String filePath = getOntologyFilePathWrite(fileName);
		
		URL fUrl = null;
		try {
			if(this.mode ==0){
				fUrl = new URL("file:/" + filePath);
			}else{
				fUrl = new URL( filePath);
			}
				
		} catch (MalformedURLException e) {
			System.out.println(e);
			e.printStackTrace();
				//throw new GenericException(e.getMessage(), "Back", "dashboard");
		}
		
		paramsOnto.put("rdfXmlURL", fUrl);
		
		Ontology ontology = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,paramsOnto);
		
		if(tokenizerPR==null){
			tokenizerPR = this.getTokenizerPR();
		}
		if(posPR == null){
			posPR = this.getPosPR();
		}
		if(morpherPR == null){
			morpherPR = this.getMorphPR();
		}
		//------------create pr param----------------
		paramsRootGaztPR = Factory.newFeatureMap(); // use default parameters
		paramsRootGaztPR.put("ontology", ontology); // gate.creole.ontology.Ontology
		paramsRootGaztPR.put("tokeniser", (DefaultTokeniser) this.tokenizerPR); // gate.creole.tokeniser.DefaultTokeniser
		paramsRootGaztPR.put("posTagger", (POSTagger) this.posPR); // gate.creole.POSTagger
		paramsRootGaztPR.put("morpher", (Morph) this.morpherPR); // gate.creole.morph.Morph
		paramsRootGaztPR.put("caseSensitive", false);//be sure
		Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, paramsRootGaztPR);
		List<String> fnames = new ArrayList<String>();
		fnames.add("Token.root");
		//-----------------------------------------------------------
		// Now we can create the Flexible Gazeteer
		paramsFlexPR = Factory.newFeatureMap();
		paramsFlexPR.put("gazetteerInst", org);
		paramsFlexPR.put("inputFeatureNames", fnames);
		
		rtn = (ProcessingResource) Factory.createResource(FLEXIBLE_GAZETTEER, paramsFlexPR);
		return rtn;
	}
	
	
	private static ProcessingResource DRInst;
	
	private ProcessingResource getDocumentResetPR() throws ResourceInstantiationException{
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(DOCUMENT_RESET, params);
		return rtn;
	}

}
