package org.ccgv.text.gate;

import gate.Factory;
import gate.FeatureMap;
import gate.Gate;
import gate.ProcessingResource;

import gate.creole.POSTagger;
import gate.creole.ResourceInstantiationException;
import gate.creole.SerialAnalyserController;
import gate.creole.gazetteer.DefaultGazetteer;
import gate.creole.gazetteer.Gazetteer;
import gate.creole.morph.Morph;
import gate.creole.ontology.Ontology;
import gate.creole.tokeniser.DefaultTokeniser;

import org.ccgv.text.gate.util.CommonMethod;
import org.ccgv.text.gate.GateExtractor.EnumProject;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;




public class GatePipelineBuilder {
	private static final String SERIAL_CONTROLLER = "gate.creole.SerialAnalyserController";
	private static final String FLEXIBLE_GAZETTEER = "gate.creole.gazetteer.FlexibleGazetteer";
	private static final String ANNIE_GAZETTEER = "gate.creole.gazetteer.DefaultGazetteer";
	private static final String ONTO_ROOT_GAZETTEER = "gate.clone.ql.OntoRootGaz";
	// private static final String ONTOLOGY_LR =
	// "gate.creole.ontology.owlim.OWLIMOntologyLR";
	private static final String ONTOLOGY_OWLIM = "gate.creole.ontology.impl.sesame.OWLIMOntology";
	private static final String JAPE_TRANSDUCER = "gate.creole.Transducer"; // alternate com.ontotext.gate.japec.JapecTransducer
	
	private static final String HASH_GAZETTEER = "com.ontotext.gate.gazetteer.HashGazetteer";//for domain dic
	
	private static final String DOCUMENT_RESET = "gate.creole.annotdelete.AnnotationDeletePR";
	private static final String REGEX_SENTENCE_SPLITTER = "gate.creole.splitter.RegexSentenceSplitter";
	private static final String ENGLISH_TOKENISER = "gate.creole.tokeniser.DefaultTokeniser";
	private static final String POS = "gate.creole.POSTagger";
	private static final String MORPHER = "gate.creole.morph.Morph";

	
	private ProcessingResource tokenizerPR ;
	private ProcessingResource posPR;
	private ProcessingResource morpherPR;
	//private URL ontoUrl;
	private Ontology ontology;
	
	private int mode;//0 stand alone 1 web
	private static final String[] PR_CLASSES = {
		"gate.creole.annotdelete.AnnotationDeletePR"      // 0 - Document Reset PR
		, "gate.creole.splitter.RegexSentenceSplitter"    // 1 - RegEx Sentence Splitter
		, "gate.creole.tokeniser.DefaultTokeniser"        // 2 - ANNIE English Tokeniser
		, "gate.creole.POSTagger"                         // 3 - ANNIE POS Tagger
		, "gate.creole.morph.Morph"                       // 4 - GATE Morphological analyser
		, FLEXIBLE_GAZETTEER                              // Flexible Gazetteer
		// , ANNIE_GAZETEER                               // ANNIE Gazetteer
		, "gate.creole.ANNIETransducer"                   // ANNIE NE Transducer
		, HASH_GAZETTEER                                   //Hash Gazetteer
		, JAPE_TRANSDUCER                                 // JAPE Transducer
		
	};
	
	
	public GatePipelineBuilder(int mode){
		this.mode = mode;
	}
	
	public SerialAnalyserController BuildControllerPipeLine(String grammarFile)
			throws ResourceInstantiationException, MalformedURLException 
	{
		CommonMethod.screenPrint("-----------mode:"+mode+"\n  grammarFile:"+grammarFile+"\n ");
		SerialAnalyserController rtn = (SerialAnalyserController) Factory.createResource(SERIAL_CONTROLLER, Factory.newFeatureMap(),Factory.newFeatureMap(), Gate.genSym());
		//---------------load basic first----------
		ProcessingResource pr;
		pr = this.getDocumentResetPR();
		rtn.add(pr);
		pr = this.getTokenizerPR();
		rtn.add(pr);
		pr = this.getSentenceSplitterPR();
		rtn.add(pr);
		//------------load POS---------------
		pr = this.getPosPR();
		rtn.add(pr);
		pr = this.getMorphPR();
		rtn.add(pr);
		//---------------load ontology ----------
		String ontoFiles = CommonMethod.getOntoFileNames(); // D:\work\webWork\apache-tomcat-8.0.0-RC5\webapps\gateCfg\ontology\carre\carreDomainMain.owl
		
		String ontoPath , ontoJapePath;
		CommonMethod.screenPrint("mode:"+mode);
		if(mode ==0){
			ontoPath =  CommonMethod.getOntologyPathWrite();//with domain
			ontoJapePath = CommonMethod.getJapePathWrite();
		}else{
			ontoPath =  CommonMethod.getOntologyPathURL();//include domainName/
			ontoJapePath = CommonMethod.getJapePathURL();
		}
		String tmpFile,tmpFilePath,tmpOntoFile,tmpJapeFile;
		ProcessingResource tmpFlexPR,tmpJapePR;
		if(ontoFiles !=null && !ontoFiles.equals("")){
			 StringTokenizer st = new StringTokenizer(ontoFiles,",");
			 CommonMethod.screenPrint("before load files:"+ontoFiles);
			 int k=0;
			 while(st.hasMoreTokens()){//for every onto file
				 CommonMethod.screenPrint("in while k="+k);
				 k++;
				 tmpFile = st.nextToken();
				 tmpOntoFile = ontoPath + tmpFile+".owl";
				 CommonMethod.screenPrint("load onto file with mode:"+mode+",tmpOntoFile:"+tmpOntoFile);
				 tmpFlexPR = getFlexibleGazetteer(tmpOntoFile);
				 rtn.add(tmpFlexPR);
				 tmpJapeFile = ontoJapePath + tmpFile+".jape";//D:\work\webWork\apache-tomcat-8.0.0-RC5\webapps\gateCfg\jape\carre\carreDomainMain.jape
				 CommonMethod.screenPrint("load onto jape file:"+tmpFile+",\n ");
				 tmpJapePR = this.getJapeTransducer(tmpJapeFile, this.ontology);
				 rtn.add(tmpJapePR);
				 CommonMethod.screenPrint("add Jar Transducer");
			 }
		}

		//-----------load jape--------------
		pr = this.getJapeTransducer(grammarFile, null);
		CommonMethod.screenPrint("load jape main file:"+grammarFile);
		rtn.add(pr);
		//--------------finished-------------------
		return rtn;
	}
	
	private static String getOntologyFilePathWrite(String fileName){
		String sProject = CommonMethod.getProject();
		CommonMethod.screenPrint("project:"+sProject+",fileName:"+fileName);
		EnumProject theProject = EnumProject.valueOf(sProject);
		
		String rtn = CommonMethod.getOntologyPathWrite();//include domain name
		
		 switch (theProject) {
		    case Carre: 
		    	rtn += fileName;//ComputerGraphics.owl
		      break;   
		    case Chic: // fall through to C
		    	rtn += fileName;
		    	break;
		    case Div: 
		    	rtn += fileName;
		      break;
		  }
		 rtn+= ".owl"; 
		 return rtn;
	}
	
	
	private ProcessingResource getDocumentResetPR() throws ResourceInstantiationException{
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(DOCUMENT_RESET, params);
		return rtn;
	}
	
	private ProcessingResource getTokenizerPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(ENGLISH_TOKENISER, params);
		return rtn;
	}
	private ProcessingResource getSentenceSplitterPR() throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(REGEX_SENTENCE_SPLITTER, params);
		return rtn;
	}
	private ProcessingResource getPosPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(POS, params);
		return rtn;
	}
	private ProcessingResource getMorphPR()throws ResourceInstantiationException {
		FeatureMap params = Factory.newFeatureMap(); // use default parameters
		ProcessingResource rtn  = (ProcessingResource) Factory.createResource(MORPHER, params);
		return rtn;
	}
	private  ProcessingResource getFlexibleGazetteer(String filePath)throws ResourceInstantiationException
	{	
		
		ProcessingResource rtn = null;
		CommonMethod.screenPrint("in getFlexibleGazetteer:"+filePath);
		//-----------create ontology param
		FeatureMap paramsOnto,paramsRGPR,paramsFlexPR;
		paramsOnto = Factory.newFeatureMap(); // use default parameters
		//String filePath = getOntologyFilePathWrite(fileName);
		//-----for test
		//this.mode =1;
		URL fUrl = null;
		try {
			if(this.mode ==0){
				fUrl = new URL("file:/" + filePath);
				CommonMethod.screenPrint("furl in mode=0 :"+fUrl.toString());
			}else{
				fUrl = new URL( filePath);
			}
				
		} catch (MalformedURLException e) {
			System.out.println(e);
			e.printStackTrace();
				//throw new GenericException(e.getMessage(), "Back", "dashboard");
		}
		CommonMethod.screenPrint("fUrl:"+fUrl.toString());
		
		paramsOnto.put("rdfXmlURL", "file://localhost/");
		
		Ontology ontology = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,paramsOnto);
		this.ontology = ontology;//ontology.getURL();
		getTokenizerPR();
		if(tokenizerPR==null){
			tokenizerPR = this.getTokenizerPR();
		}
		if(posPR == null){
			posPR = this.getPosPR();
		}
		if(morpherPR == null){
			morpherPR = this.getMorphPR();
		}
		CommonMethod.screenPrint("before create pr param");
		//------------create pr param----------------
		paramsRGPR = Factory.newFeatureMap(); // use default parameters
		paramsRGPR.put("ontology", ontology); // gate.creole.ontology.Ontology
		paramsRGPR.put("tokeniser", (DefaultTokeniser) this.tokenizerPR); // gate.creole.tokeniser.DefaultTokeniser
		paramsRGPR.put("posTagger", (POSTagger) this.posPR); // gate.creole.POSTagger
		paramsRGPR.put("morpher", (Morph) this.morpherPR); // gate.creole.morph.Morph
		paramsRGPR.put("caseSensitive", false);//be sure
		Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, paramsRGPR);
		List<String> fnames = new ArrayList<String>();
		fnames.add("Token.root");
		CommonMethod.screenPrint("after create pr param");
		//-----------------------------------------------------------
		// Now we can create the Flexible Gazeteer
		paramsFlexPR = Factory.newFeatureMap();
		paramsFlexPR.put("gazetteerInst", org);
		paramsFlexPR.put("inputFeatureNames", fnames);
		CommonMethod.screenPrint("before create flexible_gazetteer");
		rtn = (ProcessingResource) Factory.createResource(FLEXIBLE_GAZETTEER, paramsFlexPR);
		return rtn;
	}
	private ProcessingResource getJapeTransducer(String grammar,Ontology ontology)throws ResourceInstantiationException, MalformedURLException{
		if(ontology!=null){
			CommonMethod.screenPrint("___________________ontology URL:"+ontology.getURL());
		}
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		URL fUrl = null;
		if(mode==0){
			fUrl = new URL("file:/" + grammar);
		}else{
			fUrl = new URL(grammar);
		}
		params.put("grammarURL", fUrl);
		if(ontology != null ){
			params.put("ontology", ontology);
		}
		rtn = (ProcessingResource) Factory.createResource(JAPE_TRANSDUCER, params);
		return rtn;
	}
	private ProcessingResource getAnnieGazetteer()throws ResourceInstantiationException, MalformedURLException{
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		params = Factory.newFeatureMap();
		params.put("caseSensitive", false);
		rtn = (ProcessingResource) Factory.createResource(ANNIE_GAZETTEER, params);
		return rtn;
	}
	private ProcessingResource getHashGazetteer(String dicFile)throws ResourceInstantiationException, MalformedURLException{
		ProcessingResource rtn = null;
		FeatureMap params = Factory.newFeatureMap();
		params = Factory.newFeatureMap();
		//URL dicUrl = null;
		URL fUrl = null;
		if(mode==0){
			fUrl = new URL("file:/" + dicFile);
		}else{
			fUrl = new URL( dicFile);
		}
			params.put("caseSensitive", false);
			params.put("listsURL", fUrl);
			rtn = (ProcessingResource) Factory.createResource(HASH_GAZETTEER, params);
			return rtn;
	}
	
	
}
