package org.ccgv.text.gate;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;

import org.ccgv.text.gate.GateExtractor.EnumMode;
import org.ccgv.text.gate.util.CommonMethod;

import gate.*;
import gate.creole.POSTagger;
import gate.creole.ResourceInstantiationException;
import gate.creole.SerialAnalyserController;
import gate.creole.gazetteer.DefaultGazetteer;
import gate.creole.gazetteer.Gazetteer;
import gate.creole.morph.Morph;
import gate.creole.ontology.Ontology;
import gate.creole.tokeniser.DefaultTokeniser;


public class PipelineFactory {

	private static final String SERIAL_CONTROLLER = "gate.creole.SerialAnalyserController";
	private static final String FLEXIBLE_GAZETTEER = "gate.creole.gazetteer.FlexibleGazetteer";
	private static final String ANNIE_GAZETTEER = "gate.creole.gazetteer.DefaultGazetteer";
	private static final String ONTO_ROOT_GAZETTEER = "gate.clone.ql.OntoRootGaz";
	// private static final String ONTOLOGY_LR =
	// "gate.creole.ontology.owlim.OWLIMOntologyLR";
	private static final String ONTOLOGY_OWLIM = "gate.creole.ontology.impl.sesame.OWLIMOntology";
	private static final String JAPE_TRANSDUCER = "gate.creole.Transducer"; // alternate com.ontotext.gate.japec.JapecTransducer
	
	private static final String HASH_GAZETTEER = "com.ontotext.gate.gazetteer.HashGazetteer";//for domain dic
	
	private static final String DOCUMENT_RESET = "gate.creole.annotdelete.AnnotationDeletePR";
	private static final String REGEX_SENTENCE_SPLITTER = "gate.creole.splitter.RegexSentenceSplitter";
	private static final String ENGLISH_TOKENISER = "gate.creole.tokeniser.DefaultTokeniser";
	private static final String POS = "gate.creole.POSTagger";
	private static final String MORPHER = "gate.creole.morph.Morph";

	

	//private URL ontoUrl;
	private Ontology ontology;
	
	private EnumMode mode;//0 stand alone 1 web
	private static final String[] PR_CLASSES = {
		"gate.creole.annotdelete.AnnotationDeletePR"      // 0 - Document Reset PR
		, "gate.creole.splitter.RegexSentenceSplitter"    // 1 - RegEx Sentence Splitter
		, "gate.creole.tokeniser.DefaultTokeniser"        // 2 - ANNIE English Tokeniser
		, "gate.creole.POSTagger"                         // 3 - ANNIE POS Tagger
		, "gate.creole.morph.Morph"                       // 4 - GATE Morphological analyser
		, FLEXIBLE_GAZETTEER                              // Flexible Gazetteer
		// , ANNIE_GAZETEER                               // ANNIE Gazetteer
		, "gate.creole.ANNIETransducer"                   // ANNIE NE Transducer
		, HASH_GAZETTEER                                   //Hash Gazetteer
		, JAPE_TRANSDUCER                                 // JAPE Transducer
		
	};
	
	private static ProcessingResource documentResetPR;
	private static ProcessingResource tokenizerPR ;
	private static ProcessingResource sentenceSplitterPR;
	private static ProcessingResource posPR;
	private static ProcessingResource morpherPR;
	
	private static Hashtable dictionaryTable;
	
	
	public PipelineFactory(EnumMode mode){
		this.mode = mode;
	}
	
	private static boolean initDictionaries(EnumMode mode) throws ResourceInstantiationException{
		boolean rtn = false;
		if(dictionaryTable == null){
			dictionaryTable = new Hashtable();
		}
		//to check if the dic still avalible
		String strDics = CommonMethod.getDictionaries();//initialized dics 
		StringTokenizer st = new StringTokenizer(strDics,",");
		String tmpDicStr,tmpPath;
		OntoModel tmpOnto;
		ProcessingResource tmpPR;
		String ontoPath ;
		
		while(st.hasMoreTokens()){
			tmpDicStr = st.nextToken().toLowerCase();//a Dic name
			long start,end;
			 start = Calendar.getInstance().getTime().getTime();
			CommonMethod.screenPrint("load dic:"+tmpDicStr+", timmer:"+start);
			
			tmpPath = CommonMethod.getDictionaryPathOwl(tmpDicStr, mode);
			
			Object aDic = dictionaryTable.get(tmpDicStr);
			
			if(aDic == null){
				tmpOnto = getOntoModel(tmpDicStr,mode);
				dictionaryTable.put(tmpDicStr, tmpOnto);
			}
			end = Calendar.getInstance().getTime().getTime();
			long result = end -start;
			CommonMethod.screenPrint("load dic finished:"+tmpDicStr+", timmer:"+end+", diff:"+result);
		}
		rtn = true;
		return rtn;
	}
	

	//be invoked before AssembleProcessingResource or in servlet
	public static boolean initProcessingResouces(EnumMode mode){
		boolean rtn = false;
		try {
			//------------basic pr begin
			getDocumentResetPR();
			getTokenizerPR();
			getSentenceSplitterPR();
			getPosPR();
			getMorphPR();
			//-------------basic pr finished
			CommonMethod.screenPrint("in PipelineFactory basic pr finished");
			initDictionaries(mode);
		} catch (ResourceInstantiationException e) {
			// TODO Auto-generated catch block
			CommonMethod.screenPrint("in PipelineFactory.initProcessingResource:"+e.getMessage());
			e.printStackTrace();
		}
		rtn = true;
		return rtn;
	}
	//dics user choosed from interface before do analysis
	public SerialAnalyserController AssembleProcessingResource(String dics,String grammarFile,EnumMode mode){
		SerialAnalyserController rtn = null;
		try {
			rtn = (SerialAnalyserController) Factory.createResource(SERIAL_CONTROLLER, Factory.newFeatureMap(),Factory.newFeatureMap(), Gate.genSym());
			getDocumentResetPR();
			rtn.add(documentResetPR);
			getTokenizerPR();
			rtn.add(tokenizerPR);
			getSentenceSplitterPR();
			rtn.add(sentenceSplitterPR);
			getPosPR();
			rtn.add(posPR);
			getMorphPR();
			rtn.add(morpherPR);
			//---------dic------
			StringTokenizer st = null;
			if(dics!= null && !dics.equals("")){
				st = new StringTokenizer(dics,",");
			}
			String dicName,tmpPath;
			OntoModel tmpOnto;
			ProcessingResource tmpPR;
			String ontoPath ;
			OntoModel dicModel ;
			String japePath;
			ProcessingResource tmpJapePR;
			if(st!=null){
				while(st.hasMoreTokens()){
					dicName = st.nextToken().toLowerCase();//a Dic name
					dicModel = (OntoModel)dictionaryTable.get(dicName);
					if(dicModel == null){
						dicModel = getOntoModel(dicName,mode);
						dictionaryTable.put(dicName, dicModel);
					}
					rtn.add(dicModel.getFlexibleGazetteer());
					
					japePath = CommonMethod.getDictionaryPathJape(dicName,mode);
					CommonMethod.screenPrint("load onto jape file:"+japePath+",\n ");
					tmpJapePR = getJapeTransducer(japePath, dicModel.getOntoLanguageRS(),mode);
					rtn.add(tmpJapePR);
				}
			}//end of if
			//-------jape Transducer-------
			ProcessingResource pr;
			//-----------load jape--------------
			if(grammarFile!=null){
				pr = getJapeTransducer(grammarFile,null,mode);
				CommonMethod.screenPrint("load jape main file:"+grammarFile);
				rtn.add(pr);
			}
			
		} catch (ResourceInstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return rtn;
	}
	
	private static OntoModel getOntoModel(String dicName, EnumMode mode) throws ResourceInstantiationException{
		OntoModel rtn = null;
		String tmpPath = CommonMethod.getDictionaryPathOwl(dicName, mode);
		ProcessingResource pr;
		Ontology onto;
		rtn = new OntoModel();
		onto = getOntoLanguageRs(tmpPath,mode);
		pr= getFlexibleGazetteer(onto);
		//-------set OntoModel-------
		rtn.setLanguageRS(onto);
		rtn.setFlexibleGazetteer(pr);
		return rtn;
	}
	//-----------------------------------basic PRs-------------------------------
	//------------documentResetPR
	private static boolean getDocumentResetPR() throws ResourceInstantiationException{
		boolean rtn = false;
		if(documentResetPR == null)
		{
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			documentResetPR  = (ProcessingResource) Factory.createResource(DOCUMENT_RESET, params);
		}
		if(documentResetPR !=null){
			rtn = true;
		}
		return rtn;
	}
	
	//----------TokenizerPR
	private static boolean getTokenizerPR() throws ResourceInstantiationException{
		boolean rtn = false;
		if(tokenizerPR == null)
		{
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			tokenizerPR  = (ProcessingResource) Factory.createResource(ENGLISH_TOKENISER, params);
		}
		if(tokenizerPR !=null){
			rtn = true;
		}
		return rtn;
	}	
	//----------SentenceSplitter
	private static boolean getSentenceSplitterPR() throws ResourceInstantiationException{
		boolean rtn = false;
		if(sentenceSplitterPR == null)
		{
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			sentenceSplitterPR  = (ProcessingResource) Factory.createResource(REGEX_SENTENCE_SPLITTER, params);
		}
		if(sentenceSplitterPR !=null){
			rtn = true;
		}
		return rtn;
	}	
	//----------PosPR
	private static boolean getPosPR() throws ResourceInstantiationException{
		boolean rtn = false;
		if(posPR == null)
		{
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			posPR  = (ProcessingResource) Factory.createResource(POS, params);
		}
		if(posPR !=null){
			rtn = true;
		}
		return rtn;
	}	
	//----------MorphPR
	private static boolean getMorphPR() throws ResourceInstantiationException{
		boolean rtn = false;
		if(morpherPR == null)
		{
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			morpherPR  = (ProcessingResource) Factory.createResource(MORPHER, params);
		}
		if(morpherPR !=null){
			rtn = true;
		}
		return rtn;
	}		
	//--------------JapePR
	private ProcessingResource getJapeTransducer(String grammar,Ontology ontology,EnumMode mode)throws ResourceInstantiationException, MalformedURLException{
		ProcessingResource rtn = null;
		if(grammar!=null && !grammar.equals("")){
			if(ontology!=null){
				CommonMethod.screenPrint("___________________ontology URL:"+ontology.getURL());
			}
			FeatureMap params = Factory.newFeatureMap();
			URL fUrl = null;
			if(mode == EnumMode.standalong){
				fUrl = new URL("file:/" + grammar);
			}else{
				fUrl = new URL(grammar);
			}
			params.put("grammarURL", fUrl);
			if(ontology != null ){
				params.put("ontology", ontology);
			}
			rtn = (ProcessingResource) Factory.createResource(JAPE_TRANSDUCER, params);
		}
		return rtn;
	}
	
	private static ProcessingResource getFlexibleGazetteer(Ontology onto)throws ResourceInstantiationException
	{
		ProcessingResource rtn = null;
		FeatureMap paramsRGPR,paramsFlexPR;
		//-----------prepare for 
		getTokenizerPR();
		getPosPR();
		getMorphPR();
		//-------------
		paramsRGPR = Factory.newFeatureMap(); // use default parameters
		paramsRGPR.put("ontology", onto); // gate.creole.ontology.Ontology
		paramsRGPR.put("tokeniser", (DefaultTokeniser) PipelineFactory.tokenizerPR); // gate.creole.tokeniser.DefaultTokeniser
		paramsRGPR.put("posTagger", (POSTagger) PipelineFactory.posPR); // gate.creole.POSTagger
		paramsRGPR.put("morpher", (Morph) PipelineFactory.morpherPR); // gate.creole.morph.Morph
		paramsRGPR.put("caseSensitive", false);//be sure
		paramsRGPR.put("useResourceUri", false);//be sure
		Gazetteer org = (DefaultGazetteer) Factory.createResource(ONTO_ROOT_GAZETTEER, paramsRGPR);
		List<String> fnames = new ArrayList<String>();
		fnames.add("Token.root");
		//--------------
		// Now we can create the Flexible Gazeteer
		paramsFlexPR = Factory.newFeatureMap();
		paramsFlexPR.put("gazetteerInst", org);
		paramsFlexPR.put("inputFeatureNames", fnames);
		rtn = (ProcessingResource) Factory.createResource(FLEXIBLE_GAZETTEER, paramsFlexPR);
		return rtn;
	}
	
	
	//ontology file name path
		private static Ontology getOntoLanguageRs(String filePath,EnumMode mode)throws ResourceInstantiationException{
			Ontology rtn = null;
			//-----------create ontology param
					FeatureMap paramsOnto,paramsRGPR,paramsFlexPR;
					paramsOnto = Factory.newFeatureMap(); // use default parameters
					//String filePath = getOntologyFilePathWrite(fileName);
					//-----for test
					//this.mode =1;
					URL fUrl = null;
					try {
						if(mode == EnumMode.standalong){
							fUrl = new URL("file:/" + filePath);
						}else{
							fUrl = new URL( filePath);
						}
							
					} catch (MalformedURLException e) {
						System.out.println(e);
						e.printStackTrace();
							//throw new GenericException(e.getMessage(), "Back", "dashboard");
					}
					
					paramsOnto.put("rdfXmlURL", fUrl);
					CommonMethod.screenPrint("---------------------------fUrl:"+fUrl);
					rtn = (Ontology) Factory.createResource(ONTOLOGY_OWLIM,paramsOnto);
				//------------------------------
				return rtn;
		}
	
	

	
}
