package org.ccgv.text.resource.api;


import org.ccgv.text.gate.util.Binary2Text;
import org.ccgv.text.gate.util.CommonMethod;
import org.ccgv.text.model.Citation;
import org.ccgv.text.resource.api.IeeeDataInterface.SearchModel;
import org.ccgv.text.vac.BatchProcess.ResTypeEnum;
import org.dom4j.Document;
import org.dom4j.Element;

import com.google.gson.JsonArray;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;


//http://ieeexplore.ieee.org/stamp/stamp.jsp?tp=&arnumber=5543705 full pdf
//http://ieeexplore.ieee.org/gateway/ipsSearch.jsp?an=4731412 get one document

public class IeeeDataInterface extends DataInterfaceImpl {
	public String sUrl;
	
	public static String baseUrl= "http://ieeexplore.ieee.org/gateway/ipsSearch.jsp";
	public static String db = "ieee";
	//public String subUrl;
	public int iRetStart;
	
	//public String sTerm;
	public String resType;
	public final String paragraphFlag = "  <p>  ";
	public final String paragraphEndFlag = "  </p>  ";
	
	public IeeeDataInterface(ResTypeEnum resType){
		super(resType);
	}
	
	public IeeeDataInterface(){
		//subUrl = search;
		iRetStart = 0;
		iRetMax = 20;
		//sTerm = term;
		resType = "1";//0 pubmed,1 ieee
		
	}
	public void setRetMax(int retMax){
		this.iRetMax = retMax;
	}
	
	public Citation getAPaper(String pId)throws Exception{
		Citation rtn = new Citation();
		String fetchUrl = IeeeDataInterface.baseUrl +"?an=";
		fetchUrl += pId;
		URL url = new URL(fetchUrl);
	    Document document=this.parse(url);        
	    Element root = document.getRootElement();
	    
	    for ( Iterator i = root.elementIterator(); i.hasNext(); ) 
	    {
	            Element firstLevelEle = (Element) i.next();
	            rtn = this.parseADocument(firstLevelEle);
	            String tagName = firstLevelEle.getName();
	             if(tagName.equalsIgnoreCase("document")){
	            	rtn = parseADocument(firstLevelEle);
	            	
	            }
	    }
		return rtn;
	}
	
	//get a citation List
	public ArrayList<Citation> processSearch(String keyWords)throws Exception{
		 	ArrayList<Citation> rtn = null;
		 	CommonMethod.screenPrint("_______________in IeeeDataInterface processSearch: keyWords:"+keyWords);
		 	String searchUrl = IeeeDataInterface.baseUrl +"?hc="+iRetMax+"&querytext=";
			//CommonMethod.screenPrint("searchUrl:"+searchUrl);
			String term = convertKeyWords(keyWords,ResTypeEnum.ieee);
			searchUrl += term;
			//--------------
			
			URL url = new URL(searchUrl);
		    Document document=this.parse(url);        
		    Element root = document.getRootElement();
		    //sModel.idList = new ArrayList();
		    int iCount =0;
		    String tmpValue="";
		    for ( Iterator i = root.elementIterator(); i.hasNext(); ) 
		    {
		            Element firstLevelName = (Element) i.next();
		            String tagName = firstLevelName.getName();
		            if(tagName.equals("totalfound")){
		            	iCount = Integer.parseInt(firstLevelName.getText());
		            	if(iCount>0){
		            		rtn = new ArrayList<Citation>();
		            	}
		            }else if(tagName.equalsIgnoreCase("document")){
		            	Citation aPaper = parseADocument(firstLevelName);
		            	rtn.add(aPaper);
		            }
		            
		    }//ids is ready now
		    //System.out.println("searchModel:"+sModel.toString());
		    
		   // rtn = processSummary(sModel.ids);
		    
		 return rtn;
	}
	private Citation parseADocument( Element firstLevelName) {
		String tmpValue;
		String tagName;
		Citation aPaper = null;
		aPaper = new Citation();
		for( Iterator m = firstLevelName.elementIterator(); m.hasNext(); ){
			 aPaper.setResType(this.resType);
			 Element docEle = (Element) m.next();
			 tagName= docEle.getName();
			 if(tagName.equals("title")){
				 tmpValue = docEle.getText();
				 aPaper.setTitle(tmpValue);
			 }else if(tagName.equals("authors")){
				 tmpValue = docEle.getText();
				 aPaper.setAuthor(tmpValue);
			 }else if(tagName.equals("affiliations")){
				 tmpValue = docEle.getText();
				 aPaper.setAffiliation(tmpValue);
			 }else if(tagName.equals("punumber")){//?
				 tmpValue = docEle.getText();
			 }else if(tagName.equals("pubtype")){//conference/journals/books/early
				 tmpValue = docEle.getText();
				 aPaper.setPubType(tmpValue);
			 }else if(tagName.equals("py")){//publish year
				 tmpValue = docEle.getText();
				 aPaper.setYear(tmpValue);
			 }else if(tagName.equals("arnumber")){//pId
				 tmpValue = docEle.getText();
				 aPaper.setResId(tmpValue);
			 }else if(tagName.equals("pubtitle")){
				 tmpValue = docEle.getText();
				 aPaper.setPubTitle(tmpValue);
			 }else if(tagName.equals("isbn")){
				 tmpValue = docEle.getText();
				 aPaper.setIsbn(tmpValue);
			 }else if(tagName.equals("issn")){
				 tmpValue = docEle.getText();
				 aPaper.setIssn(tmpValue);
			 }else if(tagName.equals("doi")){
				 tmpValue = docEle.getText();
				 aPaper.setDoi(tmpValue);
			 }else if(tagName.equals("pdf")){
				 tmpValue = docEle.getText();
				 aPaper.setPdfLink(tmpValue);
			 }else if(tagName.equals("arnumber")){
				 tmpValue = docEle.getText();
				 aPaper.setResId(tmpValue);
			 }
			 else if(tagName.equals("controlledterms")){
				 String[] terms = getTerms(docEle);
				 aPaper.setControlledterms(terms);
			 }
			 else if(tagName.equals("uncontrolledterms")){
				 String[] terms = getTerms(docEle);
				 aPaper.setUncontrolledterms(terms);
			 }
			 else if(tagName.equals("thesaurusterms")){
				 String[] terms = getTerms(docEle);
				 aPaper.setThesaurusterms(terms);
			 }
		}//end of for
		//System.out.println("-------------in IEEE processSearch a paper:"+aPaper.toString());
		return aPaper;
	}
	private String[] getTerms(Element docEle) {
		String tmpValue;
		String tagName;
		ArrayList<String> items = new ArrayList<String>();
		 //JsonArray items = new JsonArray();
		 for( Iterator n = docEle.elementIterator(); n.hasNext(); ){
			 Element itemEle = (Element) n.next();
			 tagName= itemEle.getName();
			if(tagName.equals("term")){
				tmpValue = itemEle.getText();
				items.add(tmpValue);
			}
		 }
		// String[] rtn = new String[items.size()];
		 //String[] stringArray = Arrays.copyOf(objectArray, objectArray.length, String[].class);
		 String[] rtn = items.toArray(new String[items.size()]);
		return rtn;
	}
	public StringBuffer processFetchAbstract(String id)throws Exception{
		StringBuffer rtn = new StringBuffer();
		String fetchUrl = IeeeDataInterface.baseUrl +"?an=";
		fetchUrl += id;
		//CommonMethod.screenPrint("_______________in IeeeDataInterface processFetchAbstract:id:"+id+",fetchUrl:"+fetchUrl);
		String tmpName; 
		
		URL url = new URL(fetchUrl);
	    Document document=this.parse(url);        
	    Element root = document.getRootElement();
	    
	    for ( Iterator i = root.elementIterator(); i.hasNext(); ) 
	    {
	            Element firstLevelName = (Element) i.next();
	            String tagName = firstLevelName.getName();
	            if(tagName.equalsIgnoreCase("document")){
	            	for( Iterator m = firstLevelName.elementIterator(); m.hasNext(); ){
	            		 Element docEle = (Element) m.next();
	            		 tagName= docEle.getName();
	            		 if(tagName.equals("abstract")){
	            			 rtn.append(docEle.getText());
	            		 }
	            	}	
	            }
	    }
		return rtn;
	}
	public String process()throws Exception{
		String rtn = "";
		
		URL url = new URL(sUrl);
		HttpURLConnection conn = (HttpURLConnection)url.openConnection();
		conn.setRequestMethod("GET");
		BufferedReader br = new BufferedReader(new InputStreamReader(
	                (conn.getInputStream())));

	    String bOutput,aOutput;
	    int line=0;
	    //only read the first result
	    while ((bOutput = br.readLine()) != null) {
	    	//aOutput = Binary2Text.toText(bOutput);
	    	//gramFile.append(aOutput+"\n");
	    	//System.out.println("line"+line+":"+bOutput);
	    	line++;
	    }
	    conn.disconnect();
	    return rtn;
	}
    public static ArrayList<Citation> getIeeeCitations(){
    	ArrayList<Citation> rtn = new ArrayList();
    	return rtn;
    }
    
    public class SearchModel{
    	public int rs;
    	public int hc;
        
    }
    
    public static void main(String args[]){
    	try {
			//main0();
    		main1();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
    public static void main1(){
    	IeeeDataInterface ieeeIface = new IeeeDataInterface();
    	String pId = "4731412";
    	Citation aPaper ;
    	try {
    		aPaper = ieeeIface.getAPaper(pId);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    public static void main0() throws Exception{
    	int iMax = 100;
    	IeeeDataInterface ieeeIface = new IeeeDataInterface();
    	ieeeIface.setRetMax(iMax);
    	//ieeeIface.processSearch("gpu+graphic");
    	//http://ieeexplore.ieee.org/gateway/ipsSearch.jsp?an=4731412
    	StringBuffer sb = ieeeIface.processFetchAbstract("4731412");
    	//CommonMethod.screenPrint("abstract:"+sb);
    	
	}
	/*public static String convertKeyWords(String keyWords){
		String rtn = "";
		CommonMethod.screenPrint("in DataInterfaceImpl:"+keyWords);
		if(keyWords!=null && !keyWords.equals("")){
			if(keyWords.contains(" ")){
				rtn= keyWords.replace(" ", "%20");
			}else{
				rtn = keyWords;
			}
		}
		
		return rtn;
	}*/
}


