package org.ccgv.text.vac;

import gate.Annotation;
import gate.AnnotationSet;
import gate.Corpus;
import gate.Document;
import gate.Factory;
import gate.FeatureMap;
import gate.GateConstants;
import gate.corpora.RepositioningInfo;
import gate.creole.ResourceInstantiationException;
import gate.util.GateException;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.codec.binary.Base64;
import org.ccgv.text.clearnlp.ClearnlpDecode;
import org.ccgv.text.clearnlp.ComparatorTag;
import org.ccgv.text.clearnlp.SentenceRelationAnalysis;
import org.ccgv.text.clearnlp.SentenceRelationModel;
import org.ccgv.text.clearnlp.SentenceWordModel;
import org.ccgv.text.gate.ComparatorHtmlTag;
import org.ccgv.text.gate.ComparatorInt;
import org.ccgv.text.gate.ComparatorPhrase;
import org.ccgv.text.gate.GateExtractor;
import org.ccgv.text.gate.GateInstance;
import org.ccgv.text.gate.HtmlModel;
import org.ccgv.text.gate.PipelineFactory;
import org.ccgv.text.gate.GateExtractor.EnumMode;
import org.ccgv.text.gate.GateExtractor.EnumProject;
import org.ccgv.text.gate.GateExtractor.FileNameEnum;
import org.ccgv.text.gate.GateExtractor.SortedAnnotationList;
import org.ccgv.text.gate.HtmlModel.startEndFlagEnum;
import org.ccgv.text.gate.jape.JapeMain;
import org.ccgv.text.gate.pattern.ArgsModel;
import org.ccgv.text.gate.pattern.ArgslistModel;
import org.ccgv.text.gate.pattern.CarreRule;
import org.ccgv.text.gate.pattern.HotmapRule;
import org.ccgv.text.gate.pattern.RuleBaseWh;
import org.ccgv.text.gate.util.CommonMethod;
import org.ccgv.text.gate.util.GenerateResultFile;
import org.ccgv.text.gate.util.RandomColour;
import org.ccgv.text.model.Citation;
import org.ccgv.text.model.NewPair;
import org.ccgv.text.model.TrainingExcelModel;
import org.ccgv.text.model.db.TagModel;
import org.ccgv.text.model.db.TrainingPaperModel;
import org.ccgv.text.model.db.TrainingPhraseModel;
import org.ccgv.text.model.db.TrainingSentenceModel;
import org.ccgv.text.resource.api.DataInterfaceImpl;
import org.ccgv.text.resource.api.IeeeDataInterface;
import org.ccgv.text.resource.api.PubmedDataInterface;
import org.ccgv.text.util.CouchDBClientImpl;
import org.ccgv.text.util.DBAccess;
import org.ccgv.text.util.DocumentAccess;
import org.ccgv.text.vac.BatchProcess.ResTypeEnum;
import org.lightcouch.Attachment;
import org.lightcouch.CouchDbClient;
import org.lightcouch.Response;
import org.lightcouch.View;


public class AnalysisDomainVAC {
	
	//Analisys specified sentence from Gate by using ontology 
	//then find vac results from DB for these sentence.
	//By comparing these results we should find relation between pairs.
	public ResTypeEnum resType;
	public String pId;
	public static String paperDoc = "batch_paper";
	public static String sentenceDoc = "batch_sentence";
	public static String phraseDoc = "batch_phrase";
	public EnumMode mode = EnumMode.standalong;
	public String dics = "Carre";
	public EnumProject project  ;
	private Hashtable grpedColourTable;
	private String analysisKeyWords;
	public static boolean debugMode = false;
	
	
	
	public AnalysisDomainVAC(){
		
	}
	public AnalysisDomainVAC(String pId, ResTypeEnum resType){
		this.pId = pId;
		this.resType = resType;
		project = EnumProject.Carre;
		analysisKeyWords = "";
			
	}
	public void setRunMode(EnumMode mode){
		this.mode = mode;
	}


	
	public void setKeyWords(String analysisKeyWords){
		this.analysisKeyWords = analysisKeyWords;
	}
	public String getKeyWords(){
		return this.analysisKeyWords;
	}	
	/*public void setDocId(String docId){
		this.docId = docId;
	}
	public String getDocId(){
		return this.docId;
	}	*/
	
	//rtn result file url
	//rtn[1] doc file Id
	public String doAnalysis(){
		String rtn ="";
		String analysisKeywords = this.getKeyWords();
		
		TrainingPaperModel paperModel = findOrPreProcessAPaper(pId,resType);//@todo processPaper before Analysis it by Gate
		String docId = paperModel.getId();
		BatchProcess bProcess= new BatchProcess();
		String resultURL="";
		//------------------------------------
        JapeMain obj = new JapeMain(analysisKeywords,"userName","nodeName");//(analysisKeywords,userName,nodeName);
		String mainFileName = obj.createMainFile();//get main file formatted name
		try {
			
			ClearnlpDecode cdInstance = ClearnlpDecode.getInstance();
			GateInstance.getInstance();
			CarreRule.getInstance();
			HotmapRule.getInstance();
			
			PipelineFactory.initProcessingResouces(mode);
			CommonMethod.screenPrint("in AnalysisDomainVAC step4 initProcessingResouces");
			//----------------
			GateExtractor extObj = new GateExtractor( mainFileName,analysisKeywords,mode);//0 for stand alone
			//extObj.setTestFile(testFile);
			extObj.initSerialAnnieNew(dics);
			CommonMethod.screenPrint("in GateExtractor step5 initSerialAnnieNew");
			Corpus corpus = Factory.newCorpus("GateParser corpus");//corpus.clear();
			//-------create a file----------
			boolean flag = createCorpusDocument(corpus,docId);//dbType temp GateExtract.FileNameEnum{Pubmed,IEEE}
			// tell the pipeline about the corpus and run it
			if(flag){
				extObj.setCorpus(corpus);
				extObj.execute();
				
				StringBuffer editableContentSB = processDoc(corpus,project,paperModel);//0 useless
				//------------------find pair-------------------------
				ArrayList<NewPair> pairs = AnalysisNewPair(pId);
				if(pairs!=null&& pairs.size()>0){
					StringBuffer pairsOutputBuff = newPairsOutput(pairs);//add new PairInfo
					editableContentSB.append(pairsOutputBuff);
					CommonMethod.screenPrint("in AnalysisDomainVAC doAnalysis AnalysisNewPair result:\n"+pairs.toString());
				}
				//-----------
				//bProcess.createExcelForAPaper("carreBatch",docId,Integer.parseInt(pId));
				bProcess.createExcelForAPaper(docId);
				flag = GenerateResultFile.generateResultFileDB(editableContentSB,docId);
				if(flag){
					rtn = this.getResultFileURL(docId);
				}
				CommonMethod.screenPrint("\n ...............resultFileURL:"+rtn);
			}
		} catch (ResourceInstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (GateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}catch(Exception e){
			e.printStackTrace();
			CommonMethod.screenPrint("Exception from AnalysisDomainVAC :"+e.getMessage());
		}
		return rtn;
	}
	
	
	
	public StringBuffer processDoc(Corpus corpus,EnumProject project,TrainingPaperModel paperModel) throws IOException,Exception {
		this.project = project;
		String docId = paperModel.getId();
		String vacFlag = paperModel.getVacFlag();
		String[] domainFlag = paperModel.getDomainFlag();
		// for each document, get an XML document with the
		// person and location names added
		StringBuffer rtn = null;//return new generated result fileName
		Iterator docIter = corpus.iterator();
		int count = 0;
		//String startTagPart_1 = "<span GateID=\"";
		//String startTagPart_2 = "\" title=\"";
		//String startTagPart_3 = "\" style=\"background:Red;\">";
		//String endTag = "</span>";
		
		grpedColourTable = RandomColour.getGrpTagsWithColour(this.analysisKeyWords,project);//generate colour table for further use
		//getRequiredTags
		
		HashSet annotTypesRequired = getTagsForAnnotationSet(); //include domainTags and keywordTags
		//loop files
		while(docIter.hasNext()) {//in this case only one doc a time
			//-----------------------------processing from gate------------------
			Document doc = (Document) docIter.next();
		    AnnotationSet defaultAnnotSet = doc.getAnnotations();
		    Set<Annotation> annos = new HashSet<Annotation>(defaultAnnotSet.get(annotTypesRequired));

		    FeatureMap features = doc.getFeatures();
		    String originalContent = (String)features.get(GateConstants.ORIGINAL_DOCUMENT_CONTENT_FEATURE_NAME);
		    RepositioningInfo info = (RepositioningInfo)features.get(GateConstants.DOCUMENT_REPOSITIONING_INFO_FEATURE_NAME);
		    ++count;
		    
		    if(originalContent != null && info != null) {

		    	//Out.prln("OrigContent and reposInfo existing. Generate file...");	
		    	Iterator annosIt = annos.iterator();
		    	Annotation currAnnot;
		    	SortedAnnotationList sortedAnnotations = new SortedAnnotationList(info);
		    	int testCount = 0;
		    	long tmpAnnotStart,tmpAnnotEnd;
		    	String annType;
		    	while(annosIt.hasNext()) {
		    		currAnnot = (Annotation) annosIt.next();
		    		annType = currAnnot.getType();
		    		//---------test---------
				     tmpAnnotStart = currAnnot.getStartNode().getOffset().longValue();
				     tmpAnnotStart = info.getOriginalPos(tmpAnnotStart);
				      
				     tmpAnnotEnd = currAnnot.getEndNode().getOffset().longValue();
				     tmpAnnotEnd = info.getOriginalPos(tmpAnnotEnd, true);

				     String tmpContent = originalContent.substring((int)tmpAnnotStart,(int)tmpAnnotEnd);
				     CommonMethod.screenPrint("in SortedAnnotationList:__________***_________type:"+annType+",start:"+tmpAnnotStart+",end:"+tmpAnnotEnd+",content:"+tmpContent);
			        //----------test---------------
		    		testCount++;
		    		sortedAnnotations.addSortedExclusive(currAnnot);
		    	} // while
		    	CommonMethod.screenPrint("testCount:"+testCount);
		    	StringBuffer editableContent = new StringBuffer(originalContent);
		    	
			    TrainingExcelModel docModel = new TrainingExcelModel();
			    docModel.setPaperPortion("Abstract");
			    docModel.setPid(docId);//docId of paperModel
			    docModel.setVacFlag(vacFlag);
			    docModel.setDomainFlag(domainFlag);
			  
			    //-----------------------------processing from gate over------------------
			    //------------------save domain knowledge and present output
		    	//add tags in the content and extract knowledge from special tags
		    	boolean tagFlag =  tagAnnotation(sortedAnnotations,editableContent,info,docModel);//find sentence content and processVAC save domain sentence and ongofPairtag
		    	boolean tagNameFlag = nameTags(editableContent);//add checkbox in the result editableContent
		    	
		    	rtn = editableContent;
		    } // if - should generate

		} // while for each doc
		
		return rtn;
	}
	/*
	 * 
<OL>
	<LI>new pairs found
	<OL>
		<LI>sentence1
		<OL>
			<LI>resultTag1, phrases 1
			<LI>resultTag2, phrases 2
		</OL>
		<LI>sentence2
	</OL>
</OL>
	 * 
	 * */
	
	public StringBuffer newPairsOutput(ArrayList<NewPair> pairs){
		StringBuffer rtn = new StringBuffer();
		
		CommonMethod.screenPrint("in prepareAnalysis");
		String batchPhraseIdView = "paper/batchPhrase";
		BatchProcess bProcess = new BatchProcess();
		if(pairs != null && pairs.size()>0 ){
			try {
				
				//rtn.append("<table style='width:100%' id='t01' border='1'>");
				//rtn.append("<caption>new pairs found</caption>");
				ArrayList<TrainingPhraseModel>  phraseList;
				TrainingPhraseModel aPhrase,bPhrase;
				TrainingSentenceModel aSentenceModel;
				String sbjPhrase="",objPhrase="",predict="",resultStr="";
				
				rtn.append("<hr><OL><LI><font size='5' color='Navy'><caption>new pairs found</caption></font><OL>");
				for(NewPair aPair:pairs){//for a pair
					
					String[] ids = aPair.getPhraseIds();
					String sentenceId = aPair.getSentenceId();
					aSentenceModel = bProcess.readSentenceById(sentenceId);
					resultStr = aPair.getTagContent();//result Tag content
					
					phraseList = bProcess.readPhraseByKey(batchPhraseIdView,ids[0]);
					aPhrase = phraseList.get(0);
					if(aPhrase.getComponent().equalsIgnoreCase("SBJ")){
						sbjPhrase = aPhrase.getPhrase();
					}else if(aPhrase.getComponent().equalsIgnoreCase("OBJ")){
						objPhrase = aPhrase.getPhrase();
					}
					CommonMethod.screenPrint("aPhrase:"+aPhrase.toString());
					phraseList = bProcess.readPhraseByKey(batchPhraseIdView,ids[1]);
					bPhrase = phraseList.get(0);
					if(bPhrase.getComponent().equalsIgnoreCase("SBJ")){
						sbjPhrase = bPhrase.getPhrase();
					}else if(bPhrase.getComponent().equalsIgnoreCase("OBJ")){
						objPhrase = bPhrase.getPhrase();
					}
					//----------under line
					int idx = sbjPhrase.indexOf(resultStr);
					if(idx>0){
						sbjPhrase = sbjPhrase.substring(0, idx)+"<u>"+resultStr+"</u>"+sbjPhrase.substring(idx+resultStr.length());
					}
					idx = objPhrase.indexOf(resultStr);
					if(idx>0){
						objPhrase = objPhrase.substring(0, idx)+"<u>"+resultStr+"</u>"+objPhrase.substring(idx+resultStr.length());
					}
					//------------------
		
					if(aPhrase.getPredict().equals(bPhrase.getPredict())){
						predict = aPhrase.getPredict();
					}
					rtn.append("<LI>").append(aSentenceModel.getSentContent()).append("<OL>");
					rtn.append("<Li>");
					rtn.append("Result tag: <font size='4' color='DarkMagenta'>").append(resultStr).append("</font>&nbsp;&nbsp;=>");
					rtn.append("<font size='4' color='red'>").append(sbjPhrase).append("</font>&nbsp;");
					rtn.append("<font size='4' color='blue'><strong>").append(predict).append("</strong> </font>&nbsp;");
					rtn.append("<font size='4' color='green'>").append(objPhrase).append("</font>");
					rtn.append("&nbsp;");
					
					rtn.append("</OL>");
					
				}
				rtn.append("</OL></OL>");
				//-------------test-over--
				//weakSentenceRelationVector = fillinRelationWithRiskResult(weakSentenceList,drInstance);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}//end of if
		return rtn;
	}
	

	
	////docModel:model:pId,url,folderName,title
	private boolean tagAnnotation(SortedAnnotationList sortedAnnotations,  StringBuffer editableContentCopy, RepositioningInfo info,TrainingExcelModel docModel){
		boolean rtn = true;
		
		String endTag = "</span>";
		int annotSize = sortedAnnotations.size();
		Annotation currAnnot;
		
		String docId = docModel.getPid();//docId of docModel
		String vacFlag = docModel.getVacFlag();
		String[] domainFlag = docModel.getDomainFlag();
		
		SentenceRelationAnalysis srA ;
		BatchProcess bProcess = new BatchProcess();
		
		
		long insertPositionEnd;
		long insertPositionStart;
		String anntype,content;
		DBAccess aDB = new  DBAccess();
		String  sentenceId = null;
		//-------------------
		ArrayList<HtmlModel> htmlList = new ArrayList<HtmlModel>();
		HtmlModel tmpHtmlModel ;
		String tmpId,backGroundStr,strTagStart,strTagEnd,annTmpClass;
		//-------------------------------------------------------
		for (int i = annotSize - 1,j=0; i >= 0; --i,j++) {//Loop sentence annotation
			currAnnot = (Annotation) sortedAnnotations.get(i);
			anntype = currAnnot.getType();//type
			//--------------------deal with this annot			
			insertPositionStart = currAnnot.getStartNode().getOffset().longValue();
			insertPositionStart = info.getOriginalPos(insertPositionStart);
			
			insertPositionEnd = currAnnot.getEndNode().getOffset().longValue();
			insertPositionEnd = info.getOriginalPos(insertPositionEnd, true);
			CommonMethod.screenPrint("_______annot type"+i+" is:"+anntype+",start:"+insertPositionStart+",end:"+insertPositionEnd);
			//annotation found
			if (insertPositionEnd != -1 && insertPositionStart != -1) {
				//-----process tag
				content = editableContentCopy.substring((int)insertPositionStart,(int)insertPositionEnd);//content
				//---------style class-------
				annTmpClass = anntype;
				FeatureMap typeMap = currAnnot.getFeatures();
				String classTmp = (String)typeMap.get("type");
				
				if(classTmp!=null ){
					annTmpClass += " "+ classTmp;
				}
				//------------check if domain processed----------
				boolean containDomainFlag = false;
				if(domainFlag!=null){
					for(String aDomain:domainFlag){
						if(aDomain.equals(CommonMethod.getProject())){
							containDomainFlag = true;
							break;
						}
					}
				}
				//------------------------------------------
				//process VAC for sentence
				if(anntype.equals("Sentence")){//all sentence store vac
					if(vacFlag.equals("0")){
						sentenceId = bProcess.processVAC(docModel, insertPositionEnd,insertPositionStart, anntype, content);
					}
				}//"CA_NewRiskLinkTag"
				else if(anntype.equals(CommonMethod.getProjectPairTag())){//new risk sentence store to carre/ "CA_NewRiskLinkTag"
					CommonMethod.screenPrint("in CA_NewRiskLinkTag save sentence");
					TagModel tmpModel;
					//---------style class-------
					tmpModel = new TagModel();
					tmpModel.setContent(content);
					tmpModel.setStartId((int)insertPositionStart);
					tmpModel.setEndId((int)insertPositionEnd);	
					tmpModel.setTagName(anntype);
					if(!debugMode){
						if(!containDomainFlag){
							sentenceId = saveSentenceModel(docModel,tmpModel,CommonMethod.getProject()+"_sentence");//save one sentence //Carre_sentence
						}
					}
					addHtmlTag(endTag, currAnnot, insertPositionEnd,
							insertPositionStart, anntype, htmlList, annTmpClass);
				}//one of pair tag
				else if(anntype.equals(CommonMethod.getProjectOneOfPairTag())){//CarreResultTag
					CommonMethod.screenPrint("in OneOfPairTag save tag");
					TagModel tagModel;
					//---------style class-------
					tagModel = new TagModel("pair_Tag");
					tagModel.setContent(content);
					tagModel.setStartId((int)insertPositionStart);
					tagModel.setEndId((int)insertPositionEnd);	
					tagModel.setTagName(anntype);
					tagModel.setDocId(docModel.getPid());
					if(!debugMode){
						if(!containDomainFlag){
							sentenceId = saveTagModel(tagModel);//save one tagModel of a Doc
						}
					}
					addHtmlTag(endTag, currAnnot, insertPositionEnd,insertPositionStart, anntype, htmlList, annTmpClass);
				}//other tags
				else{//other tags add htmltag
					
					addHtmlTag(endTag, currAnnot, insertPositionEnd,insertPositionStart, anntype, htmlList, annTmpClass);
					
				}
			}//end of if
		}//end of for annotation	
		//------------sort the htmlTag list 
		ComparatorHtmlTag comparator=new ComparatorHtmlTag();
		Collections.sort(htmlList, comparator);
		//---------------insert ruleList first
		//StringBuffer sb = srA.patternAnalysis();
		//editableContentCopy.append(sb);//insert rule list from end then insert tags
		//-----------then insert tags in order-------;
		for(int i=0;i<htmlList.size();i++){//list was sorted index big first
			tmpHtmlModel = (HtmlModel)htmlList.get(i);
			editableContentCopy.insert((int) tmpHtmlModel.getIndex(), tmpHtmlModel.getHtmlTag());//insert endTag
		}
		//-------------prepare for linked sentence output--------------
		
		return rtn;
	}
	private void addHtmlTag(String endTag, Annotation currAnnot,
			long insertPositionEnd, long insertPositionStart, String anntype,
			ArrayList<HtmlModel> htmlList, String annTmpClass) {
		HtmlModel tmpHtmlModel;
		String tmpId;
		String backGroundStr;
		String strTagStart;
		String strTagEnd;
		backGroundStr = RandomColour.getRandomBackground(getTagColour(anntype));//color
		tmpId = currAnnot.getId().toString();//id
		//------------------------------------process known and unknown knowledge-----------
		//CommonMethod.screenPrint("in GateExtractor before collectTag>>>>>>>>>> insertPositionStart="+insertPositionStart+",insertPositionEnd="+insertPositionEnd+",content="+content);
		//srA.collectTags(annTmpClass,(int)insertPositionStart,(int)insertPositionEnd,content);
		//---------------------------------------------------------------------
		//---------------------------------------------------------------------
		strTagStart = "<span GateID=\""+tmpId+"\" class=\""+annTmpClass+backGroundStr;
		strTagEnd= "</span>";
		//--------add all the tags in the list----------
		//end model does not need start since end tags are same
		tmpHtmlModel = new HtmlModel();
		tmpHtmlModel.setIndex(insertPositionEnd); //end first
		tmpHtmlModel.setStartEndFlagEnum(startEndFlagEnum.END);
		tmpHtmlModel.setHtmlTag(endTag);
		htmlList.add(tmpHtmlModel);
		//start model need end index
		tmpHtmlModel = new HtmlModel();
		tmpHtmlModel.setIndex(insertPositionStart);//start
		tmpHtmlModel.setHtmlTag(strTagStart);
		tmpHtmlModel.setStartEndFlagEnum(startEndFlagEnum.START);
		tmpHtmlModel.setRefIndex(insertPositionEnd);
		htmlList.add(tmpHtmlModel);
		//---------------list add finished
	}

	
	//[0]id
	//[1]vac flag 0 non process 1 process
	private TrainingPaperModel findOrPreProcessAPaper(String pId,ResTypeEnum resType){
		TrainingPaperModel rtn=null;//docId in DB
		
		DocumentAccess docA = new DocumentAccess();
		TrainingPaperModel aPaperModel = docA.findDocumentByResIdAndResType(pId,resType);//find doc from DB
		if(aPaperModel != null){//find in DB
			rtn = aPaperModel;
		}else{
			DataInterfaceImpl dataImpl = new DataInterfaceImpl(resType).getDataInterface();
			try {
				String docId =  CommonMethod.UuidGenerator();
				TrainingPaperModel tModel = new TrainingPaperModel(docId,paperDoc);
				Citation aPaper = dataImpl.getAPaper(pId);
				aPaper.copyTo(tModel);
				tModel.setTypeName(resType.getValue());
				//---------
				CouchDBClientImpl impl = new CouchDBClientImpl();
				Response response = impl.save(tModel);
				//-------attchment
				tModel = impl.find(TrainingPaperModel.class, response.getId());
				StringBuffer abstractBuf = new StringBuffer();
				abstractBuf.append("<p>").append(aPaper.getAbstract()).append("</p>");
				//----attachment
				byte[] bytes = abstractBuf.toString().getBytes(); 
			    String data = Base64.encodeBase64String(bytes);
			    Attachment attachment = new Attachment(data,"text/html");//(data, "text/plain");	
			    //Bar bar = new Bar(); 
			    tModel.addAttachment("Abstract.html",attachment);
			    response = impl.update(tModel); //update
			    rtn = tModel;
			    //-----------
				
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		return rtn;
	}
	public  boolean createCorpusDocument(Corpus corpus,String docId) throws MalformedURLException,ResourceInstantiationException {
		//URL u = new URL("http://10.1.62.218:8080/heartFailure.txt");//http://10.1.62.239:8080/jQuery/data/gateTest.html
		//URL fileUrl = new URL("http://10.1.62.239:8080/jQuery/data/gateTest.html");
		//String fileUrl ="http://10.1.62.239:8080/jQuery/data/gateTest.html";
		boolean rtn = false;
		String sUrl = CommonMethod.getDbUrl()+docId+"/Abstract.html";
        //sUrl = "http://localhost:8080/Carre/tmpTagFile/testSentenceChic.html";//for test @todo
        //sUrl = "http://localhost:8080/jQuery/tmpTagFile/tmp_b4a167e8_9f8c_4163_817f_c82b3e48415d.html";// for test @todo
        if(sUrl !=null){
			FeatureMap params = Factory.newFeatureMap();
			params.put("sourceUrl", sUrl);//file url
			params.put("preserveOriginalContent", new Boolean(true));
			params.put("collectRepositioningInfo", new Boolean(true));
			params.put("markupAware", new Boolean(true));
			params.put("encoding", "UTF-8"); 
			
			//Out.prln("Creating doc for " + u);
			Document tmpDoc = (Document)Factory.createResource("gate.corpora.DocumentImpl", params);
			//CommonMethod.screenPrint("uuid in document:"+tmpDoc.getParameterValue("uuid"));
			corpus.add(tmpDoc);
			rtn =true;
        }
        return rtn;
	}
	/*private boolean analysisSentence(SentenceRelationModel aSentenceModel, String sentenceId){
		RuleBaseWh ruleInstance = null;
		String exeProject = EnumProject.Hotmap.toString();//CommonMethod.getProject();
		if(exeProject.equals(EnumProject.Carre.toString())){
			ruleInstance = CarreRule.getInstance();
		}else if(exeProject.equals(EnumProject.Hotmap.toString())){
			ruleInstance = HotmapRule.getInstance();
		}
		ruleInstance.Extraxtinf(aSentenceModel);
		boolean flag = savePhraseDoc(aSentenceModel,sentenceId);
		return flag;
	}*/
	//only need key from table
	private HashSet getTagsForAnnotationSet(){
		
		HashSet rtn = new HashSet(); //annotTypesRequired
		//-----------
		
		String aKey;
		 // Get all keys
	    for (Enumeration grpKeys = grpedColourTable.keys(); grpKeys.hasMoreElements();)
	    {
			Hashtable subTagTable =(Hashtable) grpedColourTable.get(grpKeys.nextElement());//domain table or keyword table
			for(Enumeration tagKeys = subTagTable.keys(); tagKeys.hasMoreElements();){
				aKey = (String)tagKeys.nextElement();
				//CommonMethod.screenPrint("in getTagsForAnnotationSet:"+aKey+",aKey.length:"+aKey.length());
				rtn.add(aKey);
			}
	    }
	    rtn.add("Sentence");
	    //-------for test--
	    //rtn.add("Lookup");
		return rtn;
	}
	private  String getResultFileURL(String uuid){
		String tagUrl = CommonMethod.getProperty(CommonMethod.DBURL);
		CommonMethod.screenPrint("in getResultFileURL:"+tagUrl);
		String rtn = tagUrl +uuid+"/"+ FileNameEnum.ResultFile.toString()+".html";
		CommonMethod.screenPrint("in getResultFileURL rtn:"+rtn);
		return rtn;
	}	
/*	private boolean savePhraseDoc(SentenceRelationModel aSentenceModel,String sentenceId){
		boolean flag = false;
		
		ArrayList<ArgslistModel> argsListModels;
		ArrayList<ArgsModel> argsModels;
		DBAccess dbA = new DBAccess();
		ArrayList<TrainingPhraseModel> tmpList = new ArrayList<TrainingPhraseModel>();		
		TrainingSentenceModel aSentence = readSentenceById(sentenceId);
		String docId = aSentence.getPid();
		TrainingPhraseModel trainPhraseModel;
		ArrayList<SentenceWordModel> Senlist=aSentenceModel.getTheSentence();//get word list
		String rawSentence = aSentenceModel.getRawSentence();
		
		if(aSentenceModel!=null){
			argsListModels = aSentenceModel.getArgslist();
			if(argsListModels!=null){
				//for(ArgslistModel aa: argsListModels){
				for(int i = 0;i<argsListModels.size();i++){
					ArgslistModel aArgsList = argsListModels.get(i);

					argsModels = aArgsList.getArgsModels();
					if(argsModels !=null){
						
						//writeTitle(2);
						//Sheet phraseTab = getWorkBook().getSheet("phraseTab");
						//--------------------
						for(int j=0;j<argsModels.size();j++){
							trainPhraseModel = new TrainingPhraseModel(CommonMethod.UuidGenerator(),"batch_phrase");
							trainPhraseModel.setLevel(aArgsList.getlevel()+"");
							trainPhraseModel.setPredict(aArgsList.getpredict());
							trainPhraseModel.setPos(aArgsList.getpos());
							
							ArgsModel aArgs = argsModels.get(j);
							trainPhraseModel.setStartId(caculatePosition(Senlist,rawSentence,aArgs.getStartID(),0));//caculatePosition(ArrayList<SentenceWordModel> Senlist,String rawSentence, int index,int type){////type start 0 or end 1
							trainPhraseModel.setEndId(caculatePosition(Senlist,rawSentence,aArgs.getEndID(),1));
							trainPhraseModel.setCore(aArgs.getCore());
							trainPhraseModel.setComponent(aArgs.getCom());
							trainPhraseModel.setArgs(aArgs.getArgs());
							trainPhraseModel.setPhrase(aArgs.getPhr());
							trainPhraseModel.setPhraseID(aArgs.getPhrID()+"");
							//CommonMethod.screenPrint("aArgs.getPhrID():"+aArgs.getPhrID()+",aArgs.getPaID():"+aArgs.getPaID());
							trainPhraseModel.setParentPhraseId(aArgs.getPaID()+"");
							trainPhraseModel.setSentenceId(sentenceId);
							trainPhraseModel.setDocId(docId);
							if(trainPhraseModel.getPredict()!=null && trainPhraseModel.getPredict().equals("analyze")){
								//CommonMethod.screenPrint("in AnalysisDomainVAC predict:"+trainModel.getPredict()+",core:"+trainModel.getCore()+",component:"+trainModel.getComponent());
							}
							tmpList.add(trainPhraseModel);
						}
					}
				}
			}
		}
		
		//----------sort----------------
		ComparatorPhrase comparator=new ComparatorPhrase();
		Collections.sort(tmpList, comparator);
		//----------------------------
		
		int curLevel=0,tmpLevel,count=0;
		int ts,te,ss,se;
		//ArrayList<TrainingPhraseModel> initSameLevelModels = new ArrayList<TrainingPhraseModel>();
		ArrayList<TrainingPhraseModel> sameLevelModels = null;
		ArrayList<TrainingPhraseModel> upperLevelModels = null;
		for(TrainingPhraseModel aTmpPhrase:tmpList){
			tmpLevel = Integer.parseInt(aTmpPhrase.getLevel());
			if(curLevel!=tmpLevel){
				upperLevelModels = sameLevelModels;
				sameLevelModels = new ArrayList<TrainingPhraseModel>();
			}	
				
			if(upperLevelModels !=null){
				for(TrainingPhraseModel aSame:upperLevelModels)
				{
						ts = aTmpPhrase.getStartId();
						te = aTmpPhrase.getEndId();
						ss = aSame.getStartId();
						se = aSame.getEndId();
						if(ss<=ts && se>=te){//contains parent
							aTmpPhrase.setParentPhraseId(aSame.getPhraseID());
						}
				}
			}
			curLevel = tmpLevel;
			
			
			sameLevelModels.add(aTmpPhrase);
			curLevel = tmpLevel;
			
			if(!debugMode){
				dbA.saveTrainingPhrase(aTmpPhrase);
			}
		}//endof for
		
		flag=true;
		return flag;
	}
*/	
	private  String getTagColour(String aTag){
		String rtn = null; //annotTypesRequired
		//-----------
		//Hashtable tagTable = GateExtractor.getGrpTagsWithColour(this.keywords) ;//only need keys
		String aKey;
		 // Get all values
	    for (Enumeration grpKeys = this.grpedColourTable.keys(); grpKeys.hasMoreElements();)
	    {
			Hashtable subTagTable =(Hashtable) this.grpedColourTable.get(grpKeys.nextElement());//domain table or keyword table
			for(Enumeration tagKeys = subTagTable.keys(); tagKeys.hasMoreElements();){
				aKey = (String)tagKeys.nextElement();
				if(aKey.equals(aTag)){
					return (String)subTagTable.get(aKey);
				}
			}
	    }
		return rtn;
	}
	//aa aa aaa bbc aaaaa aa aaa bbc aa
	private int caculatePosition(ArrayList<SentenceWordModel> Senlist,String rawSentence, int index,int type){//type start 0 or end 1
		int rtn = 0;
		int wordLength;
		String aWordRaw;
		String rawSentenceCopy;
		int sameIndex;
		String tmplateStr;
		int trueIndex;
		
		SentenceWordModel theWord = (SentenceWordModel)Senlist.get(index-1);
		rawSentenceCopy = new String( rawSentence);
		
		if(Senlist!=null){
			SentenceWordModel aWord;
			for(int i=0;i<index-1;i++){//replace same words to find the true index
				aWord = (SentenceWordModel)Senlist.get(i);
				aWordRaw = aWord.getRaw();
				wordLength = aWord.getRaw().length();
				
				if(theWord.getRaw().equals(aWordRaw)){
					char[] tmpChars = new char[wordLength];
					for(int k=0;k<wordLength;k++){
						tmpChars[k]='#';
					}
					tmplateStr = new String(tmpChars);
					sameIndex = rawSentenceCopy.indexOf(aWordRaw);
					rawSentenceCopy = rawSentenceCopy.substring(0, sameIndex)+tmplateStr+rawSentenceCopy.substring(sameIndex+wordLength);
					CommonMethod.screenPrint("rawSentenceCopy:"+rawSentenceCopy);
				}	
			}//end of for
			rtn = rawSentenceCopy.indexOf(theWord.getRaw());
			if(type==1){
				rtn = rtn+ theWord.getRaw().length();
			}
			CommonMethod.screenPrint("in AnalysisDomainVAC: caculatePosition: rtn:"+rtn+",index:"+index+",type:"+type+",rawSentence:"+rawSentence+",word:"+theWord.getRaw());
		}
		return rtn;
		
	}
	

	
	public  TrainingSentenceModel readSentenceById(String sentenceId){
		TrainingSentenceModel rtn = null;
		
		CouchDBClientImpl clientImp = new CouchDBClientImpl();
		
		String viewName = "paper/batchSentence";
		View paperView =  clientImp.view(viewName).startKey(sentenceId).endKey(sentenceId).includeDocs(true);
		//ArrayList<JsonObject> attList;
		ArrayList<TrainingSentenceModel> attList;
		attList = (ArrayList)paperView.query(TrainingSentenceModel.class);
		if(attList!=null && attList.size()==1){
			CommonMethod.screenPrint("------------------sentence start:"+sentenceId+",endkey:"+sentenceId+"----------------------------");
			rtn = (TrainingSentenceModel)attList.get(0);
			
		}
		return rtn;
	}
	//if not exist then save it
	public String saveTagModel(TagModel tModel){
		String rtn = "";
		String tagId =  CommonMethod.UuidGenerator();
		String docId = tModel.getDocId();
		String batchPairTagBydoc = "paper/batchTagPair";
		
		//ArrayList<TagModel> tags = readTag(batchPairTagBydoc,docId);
		//if(tags==null || tags.size()==0){//save
			//----------------save-----------
			CouchDBClientImpl clientImp = new CouchDBClientImpl();
			Response response = clientImp.save(tModel); //save
			tModel = clientImp.find(TagModel.class,  response.getId());
			rtn = tModel.getId();
		//}
		return rtn;
	}
	//if not exist then save it
	public String saveSentenceModel(TrainingExcelModel docModel, TagModel tagModel,String sentenceDocType){
		String rtn = "";
		DBAccess dbA = new DBAccess();
		String docId;
		String viewName;
		boolean sentenceExist;
		CouchDBClientImpl dbClient  = new CouchDBClientImpl();
		String[] domainFlag = docModel.getDomainFlag();
		if(sentenceDocType!=null && !sentenceDocType.equals("batch_sentence")){//save domain sentence not VAC sentence
			docId = docModel.getPid();//docId
			/*boolean containDomainFlag = false;
			if(domainFlag!=null){
				for(String aDomain:domainFlag){
					if(aDomain.equals(CommonMethod.getProject())){
						containDomainFlag = true;
						break;
					}
				}
			}*/
			//if(!containDomainFlag){
				String sentenceId =  CommonMethod.UuidGenerator();
				TrainingSentenceModel tsModel = new TrainingSentenceModel(sentenceId,sentenceDocType);//id,docType
				tsModel.setPid(docModel.getPid());//paperModelId
				tsModel.setPapPortion(docModel.getPaperPortion());
				tsModel.setStartId(tagModel.getStartId());//sentence index
				tsModel.setEndId(tagModel.getEndId());
				tsModel.setSenContent(tagModel.getContent());
				tsModel.setTagName(tagModel.getTagName());
				//----------------save-----------
				Response response = dbClient.save(tsModel); //save
				tsModel = dbClient.find(TrainingSentenceModel.class,  response.getId());
				updateDomainFlag(docId);
				rtn = sentenceId;
			//}//end of if
			
		}//end of if

		return rtn;
	}
	public void updateDomainFlag(String docId){
		//
		String batchPaperView = "paper/batchPaperKeyId";
		BatchProcess bProcess = new BatchProcess();

		TrainingPaperModel aPaper = bProcess.readDocById(batchPaperView,docId); //key is docId
		String[] domainFlag = aPaper.getDomainFlag();
		boolean containFlag=false;
		if(domainFlag!=null){
			for(String aDomain:domainFlag){
				if(aDomain.equals(CommonMethod.getProject())){
					containFlag = true;
				}
			}
		}
		if(!containFlag){
			if(domainFlag==null){
				domainFlag = new String[1];
				domainFlag[0]=CommonMethod.getProject();
			}else{
				domainFlag = new String[domainFlag.length+1];
				domainFlag[domainFlag.length-1]=CommonMethod.getProject();
			}
			aPaper.setDomainFlag(domainFlag);
			CouchDBClientImpl impl = new CouchDBClientImpl();
			Response res = impl.update(aPaper);
			docId = res.getId();
		}
	}
	private boolean findDomainSentence(String docId) {
		boolean rtn = false;
		String viewName;
		viewName = "paper/batchSentenceByDoc"+CommonMethod.getProject(); //paper/batchSentenceCarre
		CouchDBClientImpl dbClient  = new CouchDBClientImpl();
		View sentenceView  = dbClient.view(viewName).key(docId).includeDocs(true);
		//ArrayList<JsonObject> attList;
		ArrayList<TrainingSentenceModel> sentenceList = (ArrayList)sentenceView.query(TrainingSentenceModel.class);
		if(sentenceList!=null && sentenceList.size()>0){
			rtn = true;
		}
		return rtn;
	}	
	private boolean findDomainTag(String docId) {
		boolean rtn = false;
		String viewName;
		viewName = "paper/batchSentenceByDoc"+CommonMethod.getProject(); //paper/batchSentenceCarre
		CouchDBClientImpl dbClient  = new CouchDBClientImpl();
		View sentenceView  = dbClient.view(viewName).key(docId).includeDocs(true);
		//ArrayList<JsonObject> attList;
		ArrayList<TrainingSentenceModel> sentenceList = (ArrayList)sentenceView.query(TrainingSentenceModel.class);
		if(sentenceList!=null && sentenceList.size()>0){
			rtn = true;
		}
		return rtn;
	}	
	private boolean nameTags(StringBuffer editableContent){
		boolean rtn = false;
		int idx = editableContent.lastIndexOf("</p>");
		StringBuffer tagStrBuff = new StringBuffer();
		//-----------
		//Hashtable tagTable = GateExtractor.getGrpTagsWithColour(this.keywords) ;//only need keys
		String aGrp,aKey,colValue;
		 // Get all values from 3 tables: keywords, domain, onto
	    for (Enumeration grpKeys = this.grpedColourTable.keys(); grpKeys.hasMoreElements();)
	    {
	    	aGrp = (String)grpKeys.nextElement();
	    	tagStrBuff.append("<p><b>"+aGrp+"</b></p>");
			Hashtable subTagTable =(Hashtable) this.grpedColourTable.get(aGrp);//domain table or keyword table
			for(Enumeration tagKeys = subTagTable.keys(); tagKeys.hasMoreElements();){
				aKey = (String)tagKeys.nextElement();
				colValue =(String)subTagTable.get(aKey);
				//   <span GateID="483" style="background:#c0f1b1;">first_aldosteroneTag</span><input type="checkbox" name="tags" class = "first_aldosteroneTag" color ="#c0f1b1" checked="checked" >
				tagStrBuff.append("<input type='checkbox' name='"+aKey+"' class = '"+aKey+"' color ='#"+colValue+"' checked='checked' ><span  style='background:#"+colValue+";'>"+aKey+"</span>");
				//tagStrBuff.append("<INPUT TYPE=CHECKBOX NAME='"+aKey+"'><span style='background:#"+cValue+";font-weight:bold'>"+aKey+"</span><P>");
			}
	    }
	    //---------------
		editableContent.insert( idx, tagStrBuff.toString());
		return rtn;
		
	}
	public ArrayList<NewPair> AnalysisNewPair(String docKey){
		ArrayList<NewPair> rtn = new ArrayList<NewPair>();
		String carreSentenceView = "paper/batchSentenceByDocCarre";//carre sentence of a doc
		ArrayList<TrainingSentenceModel> carSenList;
		ArrayList<TrainingSentenceModel> batchSenList;
		ArrayList<TrainingPhraseModel> batchPhraseList;
		TrainingSentenceModel aBatchSentence;
		String carreSentenceMapping = "paper/batchSentenceCarreMapping";
		String batchPhraseBySentence = "paper/batchPhraseOfSentenceBySenId";
		String batchPairTagBydoc = "paper/batchTagPair";
		String batchPaperId = "paper/batchPaperId";
		BatchProcess bProcess = new BatchProcess();
		
		TrainingPaperModel aPaper = bProcess.readDocById(batchPaperId,docKey); //key is pId
		
		String docId = aPaper.getId();
		carSenList = readSentence(carreSentenceView,docId);
		
		ArrayList<TagModel> tags = readTag(batchPairTagBydoc,docId);
		int sentenceStart;
		if(carSenList!=null){
			String pId;
			int startId,endId;
			for(TrainingSentenceModel aCarSentence:carSenList){//loop Carre Sentence
				pId = aCarSentence.getPid();
				startId = aCarSentence.getStartId();
				endId = aCarSentence.getEndId();
				batchSenList = readSentence(carreSentenceMapping,pId,startId,endId);
				if(batchSenList!=null && batchSenList.size()==1){//get a mapping batch_Sentence
					aBatchSentence = (TrainingSentenceModel)batchSenList.get(0);
					sentenceStart = aBatchSentence.getStartId();
					batchPhraseList = readPhrase(batchPhraseBySentence,aBatchSentence.getId());//phrases belong to a sentence
					//------sort from big level to small
					sortValuePrepare(batchPhraseList);
					ComparatorInt comparator=new ComparatorInt();
					Collections.sort(batchPhraseList, comparator);
					
					if(batchPhraseList!=null ){
						for(TrainingPhraseModel aPhrase:batchPhraseList){//loop batch_phrase Sentence
							if(tags!=null){
								for(TagModel aTag:tags){
									if(fallIn(aTag,aPhrase,sentenceStart)){
										CommonMethod.screenPrint("fallIn:aTag:"+aTag.toString()+",aPhrase:"+aPhrase.toString());
										NewPair aPair = findPair(aTag,aPhrase,batchPhraseList);
										if(aPair!=null){
											aPair.setResDocId(aPaper.getPId());
											rtn.add(aPair);
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return rtn;
	}
	private NewPair findPair(TagModel aTag,TrainingPhraseModel pModel,ArrayList<TrainingPhraseModel> pList){
		NewPair rtn = null; 
		String verb = pModel.getPredict();
		 String comp = pModel.getComponent();
		 TrainingPhraseModel thePair = findOneOfPair(verb,comp,pList);
		 //----pModel and thePair are one Pair
		 if(thePair!=null){
			 rtn = new NewPair();
			 String[] phraseIds = new String[2];
			 phraseIds[0]=pModel.getId();
			 phraseIds[1]=thePair.getId();
			 rtn.setPhraseIds(phraseIds);
			 
			 rtn.setPhraseIds(phraseIds);
			 rtn.setSentenceId(pModel.getSentenceId());
			 rtn.setPredict(pModel.getPredict());
			 rtn.setTagContent(aTag.getContent());
			 
			 CommonMethod.screenPrint("find a pair:1"+pModel.toString());
			 CommonMethod.screenPrint("find a pair:2"+thePair.toString());
		 }
		return rtn;
	}
	private TrainingPhraseModel findOneOfPair(String verb,String comp,ArrayList<TrainingPhraseModel> pList){
		TrainingPhraseModel rtn=null;
		if(pList!=null){
			String compP,verbP;
			for(TrainingPhraseModel aPhrase:pList){
				compP = aPhrase.getComponent();
				verbP = aPhrase.getPredict();
				if( (compP.equals("SBJ")||compP.equals("OBJ") )&& !compP.equals(comp)){
					if(verbP.equals(verb)){
						rtn = aPhrase;
						break;
					}
				}
			}
		}
		
		return rtn;
		
	}
	private void sortValuePrepare(ArrayList<TrainingPhraseModel> phraseList){
		if(phraseList!=null ){
			for(TrainingPhraseModel aPhrase:phraseList){
				aPhrase.setValue(Integer.parseInt(aPhrase.getLevel()));
			}
		}
	}
	private boolean fallIn(TagModel aTag, TrainingPhraseModel aPhrase,int sentenceStart){
		boolean rtn = false;
		int tagS = aTag.getStartId();
		int tagE =aTag.getEndId();
		int phS = sentenceStart+aPhrase.getStartId();
		int phE = sentenceStart+aPhrase.getEndId();
		if(tagS>=phS && tagE<=phE){
			rtn = true;
		}
		CommonMethod.screenPrint("tagS:"+tagS+",tagE:"+tagE+",phS:"+phS+",phE:"+phE+", rtn:"+rtn);
		return rtn;
	}
	public ArrayList<TagModel> readTag(String viewName,Object... key){
		ArrayList<TagModel> rtn=null;
		CouchDBClientImpl clientImp = new CouchDBClientImpl();
		View paperView =  clientImp.view(viewName).key(key).includeDocs(true);
		//ArrayList<JsonObject> attList;
		rtn = (ArrayList)paperView.query(TagModel.class);
		return rtn;
	}
	public ArrayList<TrainingSentenceModel> readSentence(String viewName,Object... key){
		ArrayList<TrainingSentenceModel> rtn=null;
		CouchDBClientImpl clientImp = new CouchDBClientImpl();
		View paperView =  clientImp.view(viewName).key(key).includeDocs(true);
		//ArrayList<JsonObject> attList;
		rtn = (ArrayList)paperView.query(TrainingSentenceModel.class);
		return rtn;
	}
	public ArrayList<TrainingPhraseModel> readPhrase(String viewName,Object... key){
		ArrayList<TrainingPhraseModel> rtn=null;
		CouchDBClientImpl clientImp = new CouchDBClientImpl();
		View paperView =  clientImp.view(viewName).key(key).includeDocs(true);
		//ArrayList<JsonObject> attList;
		rtn = (ArrayList)paperView.query(TrainingPhraseModel.class);
		return rtn;
	}
	
	
	
	
	
	
	public static void testAnalysisNewPair(String docId){
		//String docId = "7cd640c2_20c9_4e3d_97ca_e4f7d6313c62";
		String key = "";//pId
		AnalysisDomainVAC anlsVac = new AnalysisDomainVAC();
		ArrayList<NewPair> pairs = anlsVac.AnalysisNewPair(key);//(docId);
	}
	private static boolean clearAllData(boolean debugMode) {
		boolean rtn = true;
		BatchProcess bProcess = new BatchProcess();
		if(!debugMode){
			bProcess.removeAllDocModel();
			bProcess.removeAllSentenceModel();
			bProcess.removeAllPhraseModel();
			bProcess.removeAllSentenceByView("paper/batchSentenceCarre");
			bProcess.removeAllByView("paper/batchTagPair");
		}
		return rtn;
	}
	public static void main(String args[]){
		//main0();
		/*String docId = "9a3b720d_8368_4cc6_b207_f695874c4b01";
		testAnalysisNewPair(docId);
		CommonMethod.screenPrint("main over");
		*/
		//batchMain();
		main1();
	}

	public static void batchMain(){
		boolean removeFlag = clearAllData(debugMode);
		String keyWords="sudden cardiac death CKD reduce" ;
		ResTypeEnum resType = ResTypeEnum.pubmed ;
		DataInterfaceImpl dataImpl = new DataInterfaceImpl(resType).getDataInterface();
	
		ArrayList<String> pairsOfDocs = new ArrayList<String>();
		try {
			ArrayList<Citation> paperList = dataImpl.processSearch(keyWords);
			String pId ;
			String resultFileURL;
			for(Citation aPaper:paperList){
				pId = aPaper.getResId();
				AnalysisDomainVAC anlsVac = new AnalysisDomainVAC(pId,resType);
				resultFileURL = anlsVac.doAnalysis();
				pairsOfDocs.add(resultFileURL);
					
				
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		CommonMethod.screenPrint("------------------batchMain is over-------------");
		
		
	}
	private static void main0() {
		String pId = "24886432";
		ResTypeEnum resType = ResTypeEnum.pubmed;
		AnalysisDomainVAC anlsVac = new AnalysisDomainVAC(pId,resType);
		anlsVac.setKeyWords("cardiovascular disease");
		try {
			boolean removeFlag = clearAllData(debugMode);
			//-------------------------
			//String result = anlsVac.doAnalysis();//result[0] url,result[1] docid
			String resultFile  = anlsVac.doAnalysis();
			CommonMethod.screenPrint("result from AnalysisDomainVAC:"+resultFile);
			BatchProcess bProcess = new BatchProcess();
			//bProcess.createExcelForAPaper( anlsVac.getDocId());//keywords,docId,index(for batch)
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private static void main1(){
	    //clearAllData(debugMode);
		String pId = "21283136";
		ResTypeEnum resType = ResTypeEnum.pubmed;
		AnalysisDomainVAC anlsVac = new AnalysisDomainVAC(pId,resType);
		anlsVac.setRunMode(EnumMode.standalong);
		String resultFileURL = anlsVac.doAnalysis();
	}
	
	
	//-----------content-------
	/*URL couchDBFile = new URL(resultFileURL);
	BufferedReader in = new BufferedReader(new InputStreamReader(couchDBFile.openStream()));
	StringBuffer sb = new StringBuffer();
	String inputLine;
	while((inputLine = in.readLine()) !=null )
		sb.append(inputLine);
	in.close();
	PrintWriter out = response.getWriter();
	out.println(sb.toString());
	//-----------on over-------------
	CommonMethod.screenPrint("in GateAnalysis uuid:"+uuid+",link:"+resultFileURL);
	*/
	
	//------------------------------output------------------------------------
	/*CommonMethod.screenPrint("--------------------------after analysis key words:"+keyWords+", the new pairs are as below:");
	int resultNumber = pairsOfDocs.size();
	CommonMethod.screenPrint("find number of pairs:"+pairsOfDocs.size()+"");
	BatchProcess bProcess = new BatchProcess();
	ArrayList<TrainingPhraseModel> phraseList;
	TrainingPhraseModel aPhrase;
	String sentenceId;
	
	String batchPhraseIdView = "paper/batchPhrase";
	if(resultNumber>0){
		for(NewPair aPair:pairsOfDocs){
			sentenceId = aPair.getSentenceId();
			TrainingSentenceModel aSentence = bProcess.readSentenceById(sentenceId);
			CommonMethod.screenPrint("@for sentence:"+aSentence.getSentContent());
			CommonMethod.screenPrint("one pair is ["+aPair.toString()+"]");
			String[] ids = aPair.getPhraseIds();
			phraseList = bProcess.readPhraseByKey(batchPhraseIdView,ids[0]);
			aPhrase = phraseList.get(0);
			CommonMethod.screenPrint("aPhrase:"+aPhrase.toString());
			phraseList = bProcess.readPhraseByKey(batchPhraseIdView,ids[1]);
			aPhrase = phraseList.get(0);
			CommonMethod.screenPrint("bPhrase:"+aPhrase.toString());
			CommonMethod.screenPrint("@for sentence over");
		}
	}
	*/
}
